import express from 'express';
import analyticController from './analytic.controller';
import { sanitize } from '../../../middleware/sanitizer';
import { jwtStrategy } from '../../../middleware/strategy';
import { userPermission } from '../../../middleware/userPermission';

export const analyticRouter = express.Router();
const rateLimit = require("express-rate-limit");
const apiLimiter = rateLimit({
    windowMs: 1 * 60 * 1000, // 1 minutes
    max: 100,
    message: "Too many accounts created from this IP, please try again after an hour"
  });
analyticRouter.route('/').get(sanitize(), jwtStrategy, userPermission('analytic-read'), analyticController.index);
analyticRouter.route('/list').get(sanitize(), jwtStrategy, userPermission('analytic-read'),analyticController.list);
analyticRouter.route('/course-detail').get(sanitize(), jwtStrategy, userPermission('analytic-course-read'),analyticController.courseData);
analyticRouter.route('/student-list').get(sanitize(), jwtStrategy, userPermission('analytic-learner-list-read'),analyticController.studentList);
analyticRouter.route('/student-quiz-result').get(sanitize(), jwtStrategy, userPermission('analytic-learner-list-read'),analyticController.studentQuizResult);
analyticRouter.route('/student-failure-reminder').post(sanitize(),apiLimiter, jwtStrategy, userPermission('analytic-write'), analyticController.resendFailureReminder);
analyticRouter.route('/lecture-list').get(sanitize(), jwtStrategy, userPermission('analytic-course-read'), analyticController.lectureList);
analyticRouter.route('/course-graph').get(sanitize(), jwtStrategy, userPermission('analytic-course-read'), analyticController.courseGraph);
analyticRouter.route('/showtotalcount').get(sanitize(), jwtStrategy, userPermission('analytic-course-total'), analyticController.showtotalcount);
analyticRouter.route('/userTrackingList').get(sanitize(),apiLimiter,  jwtStrategy, userPermission('userTrackingList-track'), analyticController.userTrackingList);
analyticRouter.route('/retryFailRemainder').get(/* sanitize(),apiLimiter,  jwtStrategy, userPermission('retryFailRemainder'), */ analyticController.retryFailRemainder);