import express from 'express';
import appsumoController from './appsumo.controller';
import { localStrategy, jwtStrategy ,rootLoginStrategy} from '../../../middleware/strategy';
import { sanitize } from '../../../middleware/sanitizer';
import { validateBody, schemas } from '../../../middleware/validator';


export const appsumoRouter = express.Router();

const rateLimit = require("express-rate-limit");
const apiLimiter = rateLimit({
    windowMs: 1 * 60 * 1000, // 1 minutes
    max: 100,
    message: "Too many accounts created from this IP, please try again after an hour"
  });
appsumoRouter.route('/').post(sanitize(), validateBody(schemas.appsumoSignUpSchema),apiLimiter, appsumoController.signup);
appsumoRouter.route('/').get(sanitize(), jwtStrategy, appsumoController.index);
appsumoRouter.route('/check').get(sanitize(),apiLimiter, jwtStrategy, appsumoController.check);
appsumoRouter.route('/redeem').get(sanitize(),apiLimiter, jwtStrategy, appsumoController.redeem);
