import bcrypt from 'bcrypt-nodejs';
import config from '../../../config';
import JWT from 'jsonwebtoken';
import { db } from '../../../models';
import mailer from '../../../mailer';
import { differentDateFormat, createChecksum ,sanitize } from './../../../functions';
import bigblue from '../../../krademy/bigbluebutton';


import { validateEmail ,makenumerickey , makealphakey } from './../../../functions';
var JWTSign = function (user, date) {
    return JWT.sign({
        iss: config.app.name,
        sub: user.id,
        iam: user.type,
        iat: date.getTime(),
        exp: new Date().setMinutes(date.getMinutes() + 60)
    }, config.app.secret);
}

export default {

    async register(req, res, next) {
        const { firstName, lastName, email, password} = req.body;
        var alias=makealphakey();
        var key =makenumerickey();
        var passwordHash = bcrypt.hashSync(password);
        db.User.findOne({ where : { email }})
        .then(user => {
            if(user)throw new RequestError('Email is already in use',409);
            return db.MetaAuth.findOne({ where : { email : email } })
        })
        .then( meta => {
            if(meta)throw new RequestError('Email is already in use',409);
            return db.Company.create({ alias, key });
        })
        .then(company=>{
            if(company)return  db.User.findOrCreate({where: {email: email}, defaults: { firstName , lastName, companyId : company.id }}).then(r=>r[0])
            throw new RequestError('Server error',401);
        })
        .then(user => {
            if(user) return db.MetaAuth.findOrCreate( { where : { email : email } ,defaults: { password : passwordHash, userId : user.id } })
            throw new RequestError('Server error',401);
        })
        .then(result => {
            if(result){
                mailer.sendRegisterationLink(email,key);
                return res.status(200).json({ email: email , key });
            }else{
                throw new RequestError('Email is already in use',409);
            }
        })
        .catch(err => {
            if(err && err.name && err.name == 'PRAException'){
                return res.status(500).json({'errors': [ err.msg ]});
            } else {
                console.log(err);
                next(err);
            }
        });
    },

    async free_trial(req, res, next) {
        const { firstName, lastName, email, password } = req.body;
        var alias=makealphakey();
        var key =makenumerickey();
        var passwordHash = bcrypt.hashSync(password);
        db.User.findOne({ where : { email }})
        .then(user => {
            if(user) throw new RequestError('This email address is already exists.',409);
            return db.Company.create({ alias, key });
        })
        .then(company=>{
            if(company) return  db.User.findOrCreate({where: {email: email}, defaults: { firstName , lastName, companyId : company.id, trial: 1 }})
            else throw new RequestError('Server error',401);
        })
        .then(user => {
            if(user) return db.MetaAuth.findOne({ where : { email : email } }).then(r =>[r, user[0]])
            else throw new RequestError('Server error',401);
        })
        .then(([metaauth, user]) => {
            if(metaauth) return db.MetaAuth.update({ password : passwordHash, userId: user.id }, { where:{ id:metaauth.id } })
            else return db.MetaAuth.findOrCreate( { where : { email : email } ,defaults: { password : passwordHash, userId : user.id, status:'active', valid:1 } })
        })
        .then(result => {
            if(result){
                mailer.freeTrial(req.body);
                mailer.sendRegisterationLink(email, key);
                mailer.mailSendToUserForFreeTrial(req.body);
                return res.status(200).json({ email: email, key: key });
            }else{
                throw new RequestError('Email is already in use',409);
            }
        })
        .catch(err => {
            next(err);
        });
    },

    async rootUserCheck(req, res) {
        if (validateEmail(req.body.email)) {
            db.MetaAuth.findOne({ where: { email: req.body.email }, include: [{ model: db.User },{ model : db.Member}]  })
                .then(meta => {
                    if(meta){
                        if(meta.User && meta.User.trial == 1){
                            // var nowDate = new Date();
                            // nowDate.setDate(nowDate.getDate()+1);
                            var nowDate = differentDateFormat(5/* , nowDate */);
                            var signupDate = new Date(meta.User.createdAt);
                            signupDate.setDate(signupDate.getDate()+14);
                            signupDate = differentDateFormat(5, signupDate);
                            if(nowDate > signupDate)
                                return res.status(401).json({ success: false, redirect: false, expired: 1 })
                        }
                        return res.status(200).json({ success: true, redirect: true, email: req.body.email , isNewUser : meta.isNewUser});
                    }
                    return res.status(401).json({ success: false, redirect: true, msg: "An Krademy account with that sign-in information does not exist. Try again or create a new account." })
                })
        }
        // else {
        //     db.Company.findOne({ where: { alias: req.body.email } })
        //         .then(company => {
        //             if (company) {
        //                 return res.status(200).json({ success: false, alias: req.body.email, redirect: true })
        //             }
        //            return res.status(200).json({ success: false, alias: req.body.email, redirect: true })
        //         })
        // }
    },

    async login(req, res) {
        if (validateEmail(req.body.email)) {
            db.MetaAuth.findOne({ where: { email: req.body.email }, include: [{ model: db.User },{ model : db.Member}]  })
            .then(meta => {
                if(meta){
                    if(meta.User && meta.User.trial == 1){
                        // var nowDate = new Date();
                        // nowDate.setDate(nowDate.getDate()+1);
                        var nowDate = differentDateFormat(5/* , nowDate */);
                        var signupDate = new Date(meta.User.createdAt);
                        signupDate.setDate(signupDate.getDate()+14);
                        signupDate = differentDateFormat(5, signupDate);
                        if(nowDate > signupDate)
                            return res.status(401).json({ success: false, redirect: false, expired: 1 })
                    }
                    var date = new Date();
                    var token = JWTSign(req.user, date);
                    res.cookie('XSRF-token', token, {
                        expire: new Date().setMinutes(date.getMinutes() + 60),
                        httpOnly: true, secure: config.app.secure
                    });
                    return res.status(200).json({ success: true, expired: 0 });
                }
                return res.status(401).json({ success: false, expired:0});
            })
        }
    },

    async changePassword(req, res,next) {
        var hash = bcrypt.hashSync(req.body.newPass);
        if(req.user.type=="root" || req.user.type=="member"){
            db.MetaAuth.findOne({ where : { id: req.user.id  } })
            .then(meta => {
                if(meta){
                    var isMatch = bcrypt.compareSync(req.body.oldPass, meta.password);
                    if(isMatch){
                       return meta;
                    }else throw new RequestError('Your current password doesn\'t matched');
                }else throw new RequestError('Invalid user received.');
            })
            .then(meta => {
                if(meta){
                    var isMatchpassword3,  isMatchpassword2 = false;
                    var isMatchpassword = bcrypt.compareSync(req.body.newPass, meta.password);
                    if(meta.password2 != null)
                        isMatchpassword2 = bcrypt.compareSync(req.body.newPass, meta.password2);
                    if(meta.password3 != null)
                        isMatchpassword3 = bcrypt.compareSync(req.body.newPass, meta.password3);
                    if(isMatchpassword || isMatchpassword2 || isMatchpassword3){
                        throw new RequestError('You don\'t use last 3 passwords');
                    }else return   db.MetaAuth.update({ password: hash, password2:meta.password,password3:meta.password2, verf_key: null, attempt: 0, valid: 1, isNewUser: 1 }, { where: { id: req.user.id }, returning: true });
                }else throw new RequestError('Server error', 500);
            })
            .then(userUpdate => {
                if (userUpdate) return res.status(200).json({ success: true })
                throw new RequestError('Server error', 500);
            })
            .catch(err => {
                next(err);
            })
        } else if(req.user.type=="employee") {
            db.Employee.update({ password: hash, verf_key: null, attempt: 0, valid: 1, isNewUser: 0 , resetPassword:0 }, { where: { id: req.user.id } })
            .then(userUpdate => {
                if (userUpdate[0]) return res.status(200).json({
                    success: true
                })
                throw new RequestError('Server error', 500);
            })
            .catch(err=>{
                next(err);
            })
        }else{
            return res.status(500).json({ Error: true });
        }
    },

    async setPassword(req, res) {
        if(req.body && req.body.token && req.body.password){
            db.MetaAuth.findOne({ where: { verf_key: req.body.token } })
            .then(result => {
                if (result) {
                    var hash = bcrypt.hashSync(req.body.password);
                    db.MetaAuth.update({ password: hash, verf_key: null, attempt: 0 ,valid: 1}, { where: { email: result.email } });
                    return res.status(200).json({ success: true });
                } 
                return res.status(500).json({ errors: ['Token has been expired, click again from email !'] });
            })
            .catch(err => {
                console.log(err);
                return res.status(500).json({ errors: ['Error Updating Password!'] });
            })
        }else return res.status(500).json({ errors: ['Invalid data received!'] });
    },

    async sendReset(req, res, next) {
        var { email } = req.body;
        const newPassword = Math.random().toString(36).replace('0.', '');
        var password = bcrypt.hashSync(newPassword);
        db.MetaAuth.findOne({ where : { email : email }})
        .then(meta => {
            if(meta) {
                if(meta.resetAttempt != null){
                    var nowDate = new Date();
                    var resetDate = new Date(meta.resetAttempt);
                    resetDate.setMinutes( resetDate.getMinutes() + 1 );
                    if(resetDate <= nowDate)
                       return db.MetaAuth.update({ isNewUser : 1, valid: 1, verf_key: newPassword, resetAttempt: new Date() }, { where: { email } }).then(r => meta)
                    else throw new RequestError('You will be able to send reset password link after 1 minute!');
                } else
                    return db.MetaAuth.update({ isNewUser : 1, valid: 1, verf_key: newPassword, resetAttempt: new Date() }, { where: { email } }).then(r => meta)
            }else{
                //throw new RequestError('If you are an existing user, we will immediately send you an email with a link to reset your password. If it is not in your inbox, Please check your spam/junk/bulk email folders.');
                return res.status(200).json({ success: true , msg: "If you are an existing user, we will immediately send you an email with a link to reset your password. If it is not in your inbox, Please check your spam/junk/bulk email folders." })
            }
        })
        .then(meta => {
            if (meta) {
                mailer.sendResetLink(email, newPassword, req.headers)
                return res.status(200).json({ success: true , msg: "Reset Link has been sent to your email!" })
            }
            throw new RequestError('Invalid Error.');
        })
        .catch(err => {
            next(err);
        })
    },

    async resetPassword(req, res ,next) {
        const { email, token, password } = req.body;
        db.MetaAuth.findOne({  where: { email: email }  })
        .then(meta => {
            if(meta) return  db.MetaAuth.findOne({  where: { email: email, verf_key: token }  })
            else throw new RequestError('Invalid email received.');
        })
        .then(result => {
            if (result) {
                var hash = bcrypt.hashSync(password);
                var isMatchpassword, isMatchpassword2,  isMatchpassword3 = false;
                if(result.password != null)
                    isMatchpassword = bcrypt.compareSync(req.body.password, result.password);
                if(result.password2 != null)
                    isMatchpassword2 = bcrypt.compareSync(req.body.password, result.password2);
                if(result.password3 != null)
                    isMatchpassword3 = bcrypt.compareSync(req.body.password, result.password3);
                if(isMatchpassword || isMatchpassword2 || isMatchpassword3){
                    throw new RequestError('Last three used passwords cannot be accepted, Please try another password.');
                }else return db.MetaAuth.update({ password: hash, password2:result.password,password3:result.password2,  verf_key: null, attempt: 0, valid: 1}, { where: { email: email } }).then(r => result);
            } else  throw new RequestError('Invalid token received.');
        })
        .then(r => {
            if(r) return res.status(200).json({ success: true });
            else throw new RequestError('Error Updating Password.');
        })
        .catch(err => {
            console.log(err);
            next(err)
        })
    },

    async profileUpdate(req, res, next){
        console.log(req.body);
        if(req.user.memberId && req.user.userId){
            db.User.update({firstName: sanitize(req.body.firstName), lastName: (req.body.lastName ?  sanitize(req.body.lastName) : null)}, {where: {id: req.user.userId, companyId: req.user.companyId}, returning:true})
            db.Member.update({firstName: sanitize(req.body.firstName), lastName: (req.body.lastName ?  sanitize(req.body.lastName) : null)}, {where: {id: req.user.memberId},returning:true})
            .then(user => {
                return db.MetaAuth.findOne({ where: { id: req.user.id }, include: [{ model: db.User, },{ model : db.Member}] });
            })
            .then(meta => {
                req.user = meta;
                return res.status(200).json({'success': true});
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
        }
        else if(req.user.type == 'member'){
            db.Member.update({firstName: sanitize(req.body.firstName), lastName: (req.body.lastName ?  sanitize(req.body.lastName) : null)}, {where: {id: req.user.memberId},returning:true})
            .then(user => {
                return db.MetaAuth.findOne({ where: { id: req.user.id }, include: [{ model: db.User, },{ model : db.Member}] });
            })
            .then(meta => {
                req.user = meta;
                return res.status(200).json({'success': true});
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
        }else{
            db.User.update({firstName: sanitize(req.body.firstName), lastName: (req.body.lastName ?  sanitize(req.body.lastName) : null)}, {where: {id: req.user.userId, companyId: req.user.companyId}, returning:true})
            .then(user => {
                return db.MetaAuth.findOne({ where: { id: req.user.id }, include: [{ model: db.User, },{ model : db.Member}] });
            })
            .then(meta => {
                req.user = meta;
                return res.status(200).json({'success': true});
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
        }
    },

    async create(req, res, next){
        var name = "Roshit+Hello";
        var data = "allowStartStopRecording=true&attendeePW=ap&autoStartRecording=false&meetingID=LMS-54552353&moderatorPW=mp&name=TEST+Roshit&record=false&voiceBridge=72505&welcome=%3Cbr%3EWelcome+to+%3Cb%3E"+name+"%3C%2Fb%3E%21";
        var checksum = createChecksum('create', data);
        data = data+'&checksum='+checksum;
        bigblue.createMeeting(data)
        .then(result => {
            return res.status(200).json({result});
        })
    },

    async joinModerator(req, res, next){
        var name = 'Roshit+Moderator';
        var data = "fullName="+name+"&meetingID=LMS-43232343&password=mp&redirect=true";
        var checksum = createChecksum('join', data);
        data = data+'&checksum='+checksum;
        bigblue.joinMeetingModerator(data)
        .then(result => {
            return res.status(200).json({result});
        })
    },

    async joinUser(req, res, next){
        var name = 'Roshit+User';
        var data = "fullName="+name+"&meetingID=LMS-43232343&password=ap&redirect=true";
        var checksum = createChecksum('join', data);
        data = data+'&checksum='+checksum;
        bigblue.joinMeetingUser(data)
        .then(result => {
            return res.status(200).json({result});
        })
    },

    async isMeetingRunning(req, res, next){
        db.Course.findOne({ where: { id: req.query.courseId}})
        .then(course => {
            if(course){
                var instructorData = JSON.parse(course.instructorDetail);
                var data = "meetingID="+req.query.meetingId;
                var checksum = createChecksum('isMeetingRunning', data);
                data = data+'&checksum='+checksum;
                return bigblue.isRunning(data).then(r => [r, instructorData.expired]);
            }
        })
        .then(([result, expired]) => {
            return res.status(200).json({running: result.response.running._text, expired: expired});
        })
    },

    async getMeetingInfo(req, res, next){
        var data = "meetingID=LMS-43232343&password=mp";
        var checksum = createChecksum('getMeetingInfo', data);
        data = data+'&checksum='+checksum;
        bigblue.allMeetingInfo(data)
        .then(result => {
            return res.status(200).json({result});
        })
    },

    async getMeetings(req, res, next){
        var checksum = createChecksum('getMeetings', '');
        var data = data+'&checksum='+checksum;
        bigblue.allMeetings(data)
        .then(result => {
            return res.status(200).json({result});
        })
    },

    async end(req, res, next){
        var data = "meetingID=LMS-43232343&password=mp";
        var checksum = createChecksum('end', data);
        data = data+'&checksum='+checksum;
        bigblue.endMeeting(data)
        .then(result => {
            return res.status(200).json({result});
        })
    },
};