import { db } from '../../../models';
import JWT from 'jsonwebtoken';
import config from '../../../config';
import { sanitize } from './../../../functions';

var JWTSign = function (user, date) {
    return JWT.sign({
        iss: config.app.name,
        sub: user.id,
        iam: user.type,
        iat: date.getTime(),
        exp: new Date().setMinutes(date.getMinutes() + 60)
    }, config.app.secret);
}
export default {
    async index(req, res, next) {
        if(req.user.companyId){
            db.Company.findOne({ where: {id: req.user.companyId}, attributes:['id', 'name', 'logo', 'referralId', 'smtpStatus', 'skipLogin'], include:[{ model:db.MetaData, where:{ key:'AD'}, attributes:['id', 'key', 'value'], required:false }] })
            .then(company => {
                if(company){
                    var isSwitchAble = { }; 
                    if(req.user.userId) isSwitchAble['root'] = true; 
                    if(req.user.memberId) isSwitchAble['member'] = true;
                    if(company.referralId) isSwitchAble['referral'] = true;
                    return res.status(200).json({logo: "logo", type: req.user.type , isSwitchAble, company:company });
                }else throw new RequestError('Company doesn\'t Exists');
                
            })
            .catch(function (error) {
                console.log(error);
                next(error)
            });
        }else{
            db.GroupMember.findOne({
                attributes:['id', 'groupId'],
                where: { memberId: req.user.memberId },
                include: [{ model: db.Group, attributes:['id', 'companyId'], include:[{ model:db.Company, attributes:['id', 'name', 'logo'] }] }]
            })
            .then(groupMember => {
                var isSwitchAble = { }; 
                if(req.user.userId) isSwitchAble['root'] = true; 
                if(req.user.memberId) isSwitchAble['member'] = true;
                return res.status(200).json({logo: "logo", type: req.user.type , isSwitchAble, company:groupMember.Group.Company });
            })
        } 
    },

    async insert(req, res, next) {
        if(req.file){
            req.file.originalname = (req.file.originalname ? sanitize(req.file.originalname) : '');
        }
        db.Company.findOne({
            where: {
                name: req.body.name
            }
        })
        .then(company => {
            if(!company){
                return db.Company.create({
                    name: req.body.name,
                    logo: req.file?req.file.filename:null,
                })
            }else throw new RequestError('Company already Exists!');
        })
        .then(company => {
            res.status(200).json({ 'success': true });
        })
        .catch(function (err) {
            console.log(err); 
            next(err);
        });
    },

    async update(req, res, next) {
        console.log(req.body);
        db.Company.findOne({  where: { id: req.params.id  }  })
        .then(company => {
            if(!company) throw new RequestError('Company doesn\'t Exists');
            else return db.Company.findOne({ where: { name: sanitize(req.body.name),  id: { $not: req.params.id } } })
        })
        .then(result => {
            if(result) throw new RequestError('Company with same name already exists');
            else{
                if(req.file){
                    req.file.originalname = (req.file.originalname ? sanitize(req.file.originalname) : '');
                    var fileExt=req.file.originalname.split('.').pop().toLowerCase();
                    if (fileExt != "jpeg" && fileExt != "jpg" && fileExt != "png") {
                        throw new RequestError('Only .png, .jpg and .jpeg format allowed!',403);
                    }
                    return db.Company.update({ name: sanitize(req.body.name), logo: (req.file.filename ? sanitize(req.file.filename) : null) },{ where: {id: req.params.id} })
                }else return db.Company.update({ name: sanitize(req.body.name) },{ where: {id: req.params.id} })
            }
        })
        .then(company => {
            if(company){
                return db.Company.findOne({  where: { id: req.params.id}, attributes:['name', 'logo']  });
            }else throw new RequestError('Error Updating Company');
        })
        .then(company => {
            res.status(200).json({ 'success': true, company: company });
        })
        .catch(function (err) {
            console.log(err); 
            next(err);
        });
    },
    async switchUser(req, res,next) {
        if(req.user && req.user.userId && req.user.memberId) {
            if(req.query.type == "r"){
                req.user.type = "root";
            }else if(req.query.type == "m"){
                req.user.type = "member";
            } else{
                throw new RequestError(' Bad request', 401);
            }
            var date = new Date();
            var token = JWTSign(req.user, date);
            res.cookie('XSRF-token', token, {
                expire: new Date().setMinutes(date.getMinutes() + 60),
                httpOnly: true, secure: config.app.secure
            });
            res.status(200).json({success : 1})
        } 
        else{
            next("Server Error")
        }
    },

    async setSmtpType(req, res,next) {
        db.Company.update({ smtpStatus: (req.query.type  == 'd' ? 'default': 'custom' )}, {where:{ id: req.user.companyId} })
        .then(r =>{
            res.status(200).json({ 'success': true });
        })
        .catch(function (err) {
            console.log(err); 
            next(err);
        });
    },

    async skipLogin(req, res,next) {
        db.Company.update({ skipLogin: req.query.type }, {where:{ id: req.user.companyId} })
        .then(r =>{
            res.status(200).json({ 'success': true });
        })
        .catch(function (err) {
            console.log(err); 
            next(err);
        });
    },
};