import express from 'express';
import { jwtStrategy } from '../../../middleware/strategy';
import companyController from './company.controller';
import multer from 'multer';
import path from 'path';
import { sanitize } from '../../../middleware/sanitizer';
import { userImplant } from '../../../middleware/auth';
import { userPermission } from '../../../middleware/userPermission';
import config from '../../../config';

var attachmentDir = path.join(config.storage.folder, config.storage.company_logo);
var storage = multer.diskStorage({
    destination: function (req, file, cb) {
      cb(null, attachmentDir)
    },
    filename: function (req, file, cb) {
      cb(null, Date.now() + path.extname(file.originalname))
    }
  })
var uploadAttachment = multer({ storage: storage, limits:{ fileSize: 10485760 },
  fileFilter: (req, file, cb) => {
    if (file.mimetype == "image/png" || file.mimetype == "image/jpg" || file.mimetype == "image/jpeg") {
      cb(null, true);
    } else {
      cb(null, false);
      return cb(new RequestError('Only .png, .jpg and .jpeg format allowed!'));
    }
  }
})

export const companyRouter = express.Router();
const rateLimit = require("express-rate-limit");
const apiLimiter = rateLimit({
  windowMs: 1 * 60 * 1000, // 15 minutes
  max: 150, 
  message: "Too many accounts created from this IP, please try again after an hour"
});
companyRouter.route('/').get(jwtStrategy, userImplant,/* userPermission('company-read'),*/ companyController.index);
companyRouter.route('/').post(sanitize(), apiLimiter, jwtStrategy,userPermission('setting-write'), uploadAttachment.single('logo'), companyController.insert);
companyRouter.route('/:id').patch(sanitize(), apiLimiter, jwtStrategy,userPermission('setting-write'), uploadAttachment.single('logo'), companyController.update);
companyRouter.route('/switchProfile').get(sanitize(),jwtStrategy/*,validateBody(schemas.sendResetPassword)*/,companyController.switchUser);
companyRouter.route('/set-smtp-type').get(sanitize(),jwtStrategy/*,validateBody(schemas.sendResetPassword)*/,userPermission('setting-write'), companyController.setSmtpType);
companyRouter.route('/skip-login').get(sanitize(),jwtStrategy/*,validateBody(schemas.sendResetPassword)*/,userPermission('setting-write'), companyController.skipLogin);
