
import { db } from '../../../models';

export default {

    async index(req, res) {
        var result = {};
        db.Member.count({ include: [{model: db.GroupMember, required:true, include: [{ model: db.Group, where: { companyId: req.user.companyId }, required:true }] }] })
        .then((data) => {
            result['users'] = data;
            return db.Group.count({where: {companyId: req.user.companyId}})
        })
        .then(data => {            
            result['departments'] = data;            
            return db.Course.count({where: {companyId: req.user.companyId, status: 'Published', courseType: 'streaming'}})
        })
        .then(data => {
            result['published_courses'] = data;
            return db.Campaign.count({where: {companyId: req.user.companyId, status: 'Completed'}})
        })
        .then(data => {
            result['campaigns'] = data;
            return db.Quiz.count({where: {companyId: req.user.companyId }});
        })
        .then(data => {
            result['quizzes'] = data;
            return db.Skill.count({where: {companyId: req.user.companyId }});
        })
        .then(data => {
            result['skills'] = data;
            return db.Campaign.findAll({where: {companyId: req.user.companyId }, attributes:['name', 'createdAt'],include:[ {model: db.Simulation, attributes: ['type'] } ], limit: 5, order: [ ['id', 'DESC'] ]})
        })
        .then(data => {
            result['recent_campaigns'] = data;
            res.status(200).json({'counts': result});
        })
        .catch(err => {
            console.log(err);
            throw new RequestError('Fetching Error Counts');
        })
    },

    async campaignTypes(req, res) {
        var result = {};
        db.Campaign.count({where: {companyId: req.user.companyId, status: 'Completed' }, includes:[{ model: db.Simulation}] })
        .then(data => {
            result['total'] = data;
            return db.Simulation.count({ where: { type: 'course' }, include:[ {model: db.Campaign, where: {companyId: req.user.companyId, status: 'Completed' } } ]})
        })
        .then(data => {
            result['courses'] = data;
            return db.Simulation.count({ where: { type: 'quiz' }, include:[ {model: db.Campaign, where: {companyId: req.user.companyId, status: 'Completed' } } ]})
        })
        .then(data => {
            result['quizzes'] = data;
            return db.Simulation.count({ where: { type: 'announcement' }, include:[ {model: db.Campaign, where: {companyId: req.user.companyId, status: 'Completed' } } ]})
        })
        .then(data => {
            result['announcements'] = data;
            res.status(200).json({'counts': result});
        })
        .catch(err => {
            console.log(err);
            throw new RequestError('Fetching Error Counts');
        })
    },

    async graph(req, res) {
        var tempDate = new Date();
        tempDate.setMonth(tempDate.getMonth() - 7);
        db.SessionCourseLectureTracking.findAll({  
            attributes: [ [db.sequelize.fn('SUM', db.sequelize.col('SessionCourseLectureTracking.duration')), 'total'], [db.sequelize.literal("CASE month(`SessionCourseLectureTracking`.`createdAt`) WHEN 1 THEN 'Jan' WHEN 2 THEN 'Feb' WHEN 3 THEN 'Mar' WHEN 4 THEN 'Apr' WHEN 5 THEN 'May' WHEN 6 THEN 'Jun' WHEN 7 THEN 'Jul' WHEN 8 THEN 'Aug' WHEN 9 THEN 'Sept' WHEN 10 THEN 'Oct' WHEN 11 THEN 'Nov' WHEN 12 THEN 'Dec' END"), 'months']],
            where: { createdAt: { $gte: tempDate } },
            order: ['createdAt'],
            group: ['months'],
            include: [{
                model: db.Session,
                where : { companyId : req.user.companyId}
            }]
        })
        .then(graph => {
            graph = json(graph);
            var finalData = {};
            for (var i = 0; i < graph.length; i++) 
                finalData[graph[i]['months']] = graph[i].total;
            return res.status(200).json({ graph: finalData });
        })
        .catch(err => {
            console.log(err);
            throw new RequestError('Fetching Error Counts');
        })
    },

    async recentActivity(req, res){
        var condition;
        var userId = req.user.User.id;
        var type = req.user.type;
        var companyId = req.user.companyId;
        if(type == 'root') condition = { companyId :companyId }
        else condition = { userId: userId, companyId:companyId }
        db.RecentActivity.findAll({where : condition, limit: 3, offset: 0, order : [['createdAt', 'DESC']]})
        .then(r => {
            if(r) return res.status(200).json({data: r});
            else throw new RequestError('No Activity Found');
        })
        .catch(err => {
            console.log(err);
            throw new RequestError('Fetching Error Counts');
        })
    },

    async topCourses(req, res){
        db.Course.findAll({
            attributes: ["name", [db.sequelize.literal("(select count(cc.`id`) from CampaignCourses cc INNER JOIN Courses c ON c.`id` = cc.`courseId` where c.companyId = "+req.user.companyId+" )"), "campaignCount"], [db.sequelize.literal("(SELECT COUNT(*) FROM CampaignCourses WHERE CampaignCourses.courseId = Course.id)"), "courseCount"]],
            where :{ status: "Published", companyId: req.user.companyId },
            order: [[db.Sequelize.literal("courseCount"), "DESC"]],
            offset :0,
            limit : 5,
        })
        .then(courseList =>{
            if(courseList) return res.status(200).json({ data : courseList });
            else throw new RequestError('No Courses Found');
        })
        .catch(err => {
            console.log(err);
            throw new RequestError('Fetching Error Counts');
        })
    },

    async avtivityLog(req ,res, next){
        var searchDate = '';
        var startDate = '';
        var enddate ='';
        var moment = require('moment')
        if(req.query.activityDate == 'today'){
            startDate = moment().format('YYYY-MM-DD');
            searchDate=[ startDate+' 00:00:00', startDate+' 23:59:59'];
        }
        if(req.query.activityDate == 'agoday'){
            startDate = moment().subtract(1, "days").format("YYYY-MM-DD");
            searchDate=[ startDate+' 00:00:00', startDate+' 23:59:59'];
        }
        if(req.query.activityDate == 'lastweek'){
            startDate = moment().format('YYYY-MM-DD');
            enddate = moment().subtract(1, 'weeks').format('YYYY-MM-DD');
            searchDate=[enddate+' 00:00:00', startDate+' 23:59:59'];
        }
        if(req.query.activityDate =='lastMonth'){
            startDate = moment().format('YYYY-MM-DD');
            enddate = moment(startDate).subtract(1, 'months').format('YYYY-MM-DD');
            searchDate=[enddate+' 00:00:00', startDate+' 23:59:59'];
        }
        if(req.query.activityDate =='threeMonth'){
            startDate = moment().format('YYYY-MM-DD');
            enddate = moment(startDate).subtract(3, 'months').format('YYYY-MM-DD');
            searchDate=[enddate+' 00:00:00', startDate+' 23:59:59'];
        }
        if(req.query.activityDate =='sixMonth'){
            startDate = moment().format('YYYY-MM-DD');
            enddate = moment(startDate).subtract(6, 'months').format('YYYY-MM-DD');
            searchDate=[enddate+' 00:00:00', startDate+' 23:59:59'];
        }
        if(req.query.activityDate =='year'){
            startDate = moment().format('YYYY-MM-DD');
            enddate = moment(startDate).subtract(1, 'year').format('YYYY-MM-DD');
            searchDate=[enddate+' 00:00:00', startDate+' 23:59:59'];
        }
        var condition = { companyId: req.user.companyId };
        if(req.query.type != 'all')
            condition['type'] = req.query.type;
        if(req.query.activityDate != 'all')
            condition['createdAt'] = { $between: searchDate };
        db.RecentActivity.findAll({
            attributes: ['id', 'userId','companyId','type','text','createdAt','updatedAt'],
            where :condition,
            include: [{ 
                model: db.User, 
                attributes: ['id','firstName','lastName'], 
                required: true 
            }],
            offset: (req.query.limit != 10 ? parseInt(req.query.limit) : 0), 
            limit: 10, 
            order : [['createdAt', 'DESC']]
        })
        .then(r => {
            if(r) return res.status(200).json({data: r});
            else throw new RequestError('No Activity Found');
        })
        .catch(err => {
            console.log(err);
            next(err);
        })
    }
}