import { db } from '../../../models';
import { queue } from '../../../kue';
import AD from '../../../krademy/AD';
import bcrypt from 'bcrypt-nodejs';
import { CheckIsValidDomain } from '../../../functions';
import { sanitize } from './../../../functions';

function makeid() {
    var result = '', characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789', charactersLength = 62;
    for ( var i = 0; i < 30; i++ ) { result += characters.charAt(Math.floor(Math.random() * charactersLength)) }
    return result;
}

var addUpdateGroup = function(member, groupId){
    return new Promise((resolve, reject) => {
        if(member.mode == 'add'){
            db.Member.findOne({ where: { email: member.email }  })
            .then(memberDetail => {
                if(!memberDetail){
                    return db.Member.create({
                        firstName: member.firstName? sanitize(member.firstName):null,
                        lastName: member.lastName? sanitize(member.lastName):null,
                        email: member.email? sanitize(member.email):null,
                        phone: member.phone? sanitize(member.phone):null,
                        designation: member.designation? sanitize(member.designation):null,
                        code :makeid(),
                    })
                }else{
                    return db.Member.update({
                        firstName: member.firstName? sanitize(member.firstName):null,
                        lastName: member.lastName? sanitize(member.lastName):null,
                        phone: member.phone? sanitize(member.phone):null,
                        designation: member.designation? sanitize(member.designation):null,
                    }, {
                        where: {email: member.email}
                    }).then(r => memberDetail)
                }
            })
            .then(memberDetail => {
                resolve(memberDetail);
            })
            .catch(err =>{
                reject(err);
            })
        }else if(member.mode == 'update'){
            db.Member.findOne({ where: { email: member.email} })
            .then(memberDetail => {
                if(!memberDetail){
                    return db.Member.create({
                        firstName: member.firstName? sanitize(member.firstName):null,
                        lastName: member.lastName? sanitize(member.lastName):null,
                        email: member.email? sanitize(member.email):null,
                        phone: member.phone? sanitize(member.phone):null,
                        designation: member.designation? sanitize(member.designation):null,
                        code :makeid(),
                    })
                }else{
                    return db.Member.update({
                            firstName: member.firstName? sanitize(member.firstName):null,
                            lastName: member.lastName? sanitize(member.lastName):null,
                            email: member.email? sanitize(member.email):null,
                            phone: member.phone? sanitize(member.phone):null,
                            designation: member.designation? sanitize(member.designation):null,
                        }, {
                            where: {id: memberDetail.id}
                    }).then(r => memberDetail);
                }
            })
            .then(memberDetail =>{
                if(memberDetail && memberDetail.id != member.id)
                    return db.GroupMember.destroy({ where: { groupId: groupId, memberId: member.id } }).then(r => memberDetail)
                else return memberDetail;
            })
            .then(memberDetail => {
                return db.MetaAuth.findOne({where:{ email: memberDetail.email  } }).then(r => [r, memberDetail])
            })
            .then(([metaAuths, memberDetail]) => {
                if(!metaAuths) 
                    return db.MetaAuth.create({ email: memberDetail.email, memberId: memberDetail.id, valid: 1, isNewUser: 1 }).then(r => memberDetail)
                else return db.MetaAuth.update({ email: memberDetail.email  }, {where:{ id: metaAuths.id } }).then(r => memberDetail)
            })
            .then(memberDetail => {
                return db.Session.findOne({  where:{ memberId: memberDetail.id } }).then(r => [r, memberDetail])
            })
            .then(([session, memberDetail]) => {
                if(session){
                    return db.Session.update({  
                        firstName: member.firstName? sanitize(member.firstName):null,
                        lastName: member.lastName? sanitize(member.lastName):null,
                        email: member.email? sanitize(member.email):null,
                        phone: member.phone? sanitize(member.phone):null, 
                    },{
                        where:{ memberId: memberDetail.id }
                    }).then(r => memberDetail)
                }else return memberDetail;
            })
            .then(memberDetail => {
                resolve(memberDetail);
            })
            .catch(err =>{
                reject(err);
            })
        }
    })
}


export default {
    async index(req, res, next) {
        let limit = 10;
        let offset = 0;
        let sort = ['createdAt', 'DESC'];
        let status = '';
        let search = '%%';
        if(req.query.limit != undefined){
            limit = parseInt(req.query.limit);
        }
        if(req.query.search != undefined){
            search = '%'+req.query.search+'%';
        }
        if(req.query.page != undefined){
            var page = req.query.page;
            if(page < 1)
                page = 1;
        }
        if(req.query.sort){
            if(req.query.sort == 'name'){
                sort = ['name'];
            }else if(req.query.sort == 'completed'){
                status = req.query.sort;
            }else if(req.query.sort == 'pending'){
                status = req.query.sort;
            }else if(req.query.sort == 'error'){
                status = 'failed';
            }else if(req.query.sort == 'all'){
                status = '';
            }
        }
        db.Group.findAndCountAll({
            where: {  name: {like: search}, status: (status == '' ? ['completed', 'pending', 'failed']: status ), companyId: req.user.companyId }
        })
        .then((data) => {
            let pages = Math.ceil(data.count / limit);
            var selectOptions = null; 
            if(!page){
                selectOptions = {
                    attributes: ['id', 'name', 'createdAt', 'status', 'parentId', 'userCount'],
                    where: {  status: "completed", companyId: req.user.companyId},
                    include:[{ model: db.GroupSkill, include:[{ model:db.Skill }] }],
                    order: [
                        ['id', 'DESC'],
                    ],
                };
            }else{
                offset = limit * (page - 1);
                selectOptions = {
                    attributes: ['id', 'name', 'status', 'createdAt', 'parentId', 'userCount'],
                    where: { name: {like: search},  status: (status == '' ? ['completed', 'pending', 'failed']: status ), companyId: req.user.companyId},
                    include:[{ model: db.GroupSkill, attributes:['groupId', 'skillId'], include:[{ model:db.Skill, attributes:['name'],}] }],
                    limit: limit,
                    offset: offset,
                    order: [sort],
                }
            }
            db.Group.findAll(selectOptions)
            .then((groups) => {
                if(page)
                    res.status(200).json({'groups': groups, 'count': data.count, 'pages': pages});
                else{
                    res.status(200).json({'groups': groups});
                }
            })
            .catch(function (err) {
                next(new RequestError('Error fetching Departments'))
            });
        })
        .catch(function (err) {
            next(new RequestError('Error fetching Departments'))
        });
    },

    async fetchAllExistGroups(req, res, next) {
        db.Group.findAll({ where : { $not: { id: req.query.id }, companyId:req.user.companyId }, order : [['id', 'DESC']] })
        .then(groups => {
            return res.status(200).json({groups: groups });
        })
        .catch(err => {
            next(err)
        })
    },

    async find(req, res, next){
        db.Group.findOne({
            attributes: ['id', 'name', 'createdAt', 'status', 'parentId'],
            where: { id: req.params.id, companyId: req.user.companyId },
            include:[{ model:db.GroupSkill, attributes:['groupId', 'skillId'] }]
        })
        .then(group => {
            if(group){
                if(group.parentId > 0){
                    return db.Group.findOne({
                        attributes: ['id', 'name'],
                        where :{ id: group.parentId, companyId: req.user.companyId},
                        include:[{ model:db.GroupSkill, attributes:['groupId', 'skillId'] }]
                    }).then(r => [group, r]);
                }else return [group, false];
            }else throw new RequestError('Department doesn\'t exists!');
        })
        .then(([group, parentGroup]) => {
            if(group){
                res.status(200).json({group: group, parentGroup: parentGroup});
            }else throw new RequestError('Department doesn\'t exists!');
        })
        .catch(err => {
           next(err)        
        })
    },

    async fetchAll(req, res, next) {
        if(req.params && req.params.id){
            db.Group.findAll({ 
                attributes: ['id', 'name', [db.sequelize.literal('(SELECT COUNT(`GroupMembers`.`id`) FROM `GroupMembers` WHERE `GroupMembers`.`groupId` = `Group`.`id`)'), 'members']],
                where: { parentId:req.params.id, companyId: req.user.companyId} })
            .then(groups => {
                if(groups)
                    return res.status(200).json({groups: groups});
                else throw new RequestError('Department doesn\'t exists!');
            })
            .catch(err => {
                next(err)
            })
        }else next(new RequestError('Invalid data received!'));
    },

    async delete(req, res, next){
        db.Group.findOne({
            attributes: ['id', 'name', 'createdAt'],
            where: { id: req.params.id }
        })
        .then(group => {
            if(group){
                return db.GroupMember.destroy({ where: { groupId: req.params.id }});
            }else throw new RequestError('Department doesn\'t exists!');
        })
        .then(r => {
            return db.Group.destroy({where: {id: req.params.id}});
        })
        .then(r => {
            if(r){
                res.status(200).json({success: true});
            }else throw new RequestError('Error deleting Department!');
        })
        .catch(err => {
            next(err);
        })
    },

    async create(req, res, next) {
        var uaParser = require('ua-parser-js');
        var ip = req.headers["x-real-ip"] || req.ip;
        var detail = {};
        detail.user_agent = req.headers['user-agent'];
        detail.systemDetail = uaParser(req.headers['user-agent']);
        detail.ip = ip;
        var info = { info: detail};
        var userId = req.user.User.id;
        var type = 'department';
        var metaData = JSON.stringify(info);
        var companyId = req.user.companyId;
        var username = req.user.User.firstName+' '+req.user.User.lastName;
        var text = sanitize(req.body.name)+' department created by '+username;
        db.Group.findOne({ where: { name: req.body.name, companyId: req.user.companyId} })
        .then(group => {
            if(group) throw new RequestError('Department with same name already exists!');
            else {
                if(req.file){
                    req.file.originalname = (req.file.originalname ? sanitize(req.file.originalname) : '');
                    var fileExt=req.file.originalname.split('.').pop().toLowerCase();
                    if (fileExt != "csv") {
                        throw new RequestError('Only .csv file allowed!',403);
                    }
                }
                db.RecentActivity.create({userId, companyId, type, text, metaData});
                return db.Group.create({ name: sanitize(req.body.name), status: "pending", parentId: req.body.parentId, companyId : req.user.companyId });
            }  
        })
        .then(group => {
            if(req.body.skills){
                var skills = [];
                var skill = req.body.skills.split(',');
                for(var i=0;i< skill.length; i++){
                    skills.push({
                        'groupId' : group.id ,
                        'skillId' : skill[i]
                    })
                }
                return db.GroupSkill.bulkCreate(skills).then(r => group);  
            }else return group;
        })
        .then(group => {
            if(req.file && req.file.filename){
                queue.create('create-group', {
                    groupId: group.id,
                    companyId: req.user.companyId,
                    fileName: req.file.filename,
                }).save();
            }
            return group;
        })
        .then(group => {
         //   console.log(req.body.members)
            var allTasks = [], members = JSON.parse(req.body.members);
            var managerId = null;
            for(var i=0;i<members.length;i++){
                allTasks.push(new Promise((resolve, reject)=> {
                    var member = members[i];
                    db.Member.findOne({  where: { email: member.email } })
                    .then(checkMember => {
                        if(!checkMember){
                            return db.Member.create({
                                firstName: member.firstName? sanitize(member.firstName):null,
                                lastName: member.lastName? sanitize(member.lastName):null,
                                email: member.email? sanitize(member.email):null,
                                phone: member.phone? sanitize(member.phone):null,
                                designation: member.designation? sanitize(member.designation):null,
                                code :makeid(),
                            });
                        }else{
                            return db.Member.update({
                                firstName: member.firstName? sanitize(member.firstName):null,
                                lastName: member.lastName? sanitize(member.lastName):null,
                                email: member.email? sanitize(member.email):null,
                                phone: member.phone? sanitize(member.phone):null,
                                designation: member.designation? sanitize(member.designation):null,
                            }, { where: { email: member.email }
                            }).then(r => checkMember)
                        }
                    })
                    .then(checkMember => {
                        queue.create('assign-policy', { id : checkMember.id, modules : ['home','learner','course-analytics'] }).save();
                        if(member.isManager === 'true') 
                            managerId =  checkMember.id;
                        db.GroupMember.create({
                            groupId: group.id,
                            memberId: checkMember.id,
                            isManager: (member.isManager === 'true' ? 1 : 0),
                        });
                        resolve(true)
                    })
                    .catch(err => {
                        reject(err);
                    })
                }))
            }
            return Promise.all(allTasks).then(r => [group, members.length, managerId]);
        })
        .then(([group, memberLength, managerId]) => {
            if(!req.file){
                return db.Group.update({ status: 'completed', userCount: memberLength, managerId: managerId }, {  where: {id: group.id, companyId: req.user.companyId} }).then(r => group);
            } else {
                return group;
            }
        })
        .then(status => {
            res.status(200).json({'success': true});
        })
        .catch(err => {
            next(err);
        })
    },

    async update(req, res, next) {
        var addedCount = 0;
        try{
            var mainGroup =  db.Group.findOne({ where: { id: req.params.id, companyId: req.user.companyId } })
            if(mainGroup){
                db.Group.findOne({  where: { name: req.body.name, companyId: req.user.companyId,  id: { $not: req.params.id }} })
                .then(r => {
                    if(!r){
                        if(req.file && req.file.filename){
                            req.file.originalname = (req.file.originalname ? sanitize(req.file.originalname) : '');
                            var fileExt=req.file.originalname.split('.').pop().toLowerCase();
                            if (fileExt != "csv") {
                                throw new RequestError('Only .csv file allowed!',403);
                            }
                            return db.Group.update({ status: 'pending', parentId:req.body.parentId }, { where: {id: mainGroup.id} }).then(r => mainGroup);
                        }else 
                            return mainGroup;
                    } else throw new RequestError('Department with same name already exists!');
                })
                .then(group => {
                    if(req.body.skills)
                        return db.GroupSkill.destroy({where : {groupId : group.id}}).then(r => group)  
                    else return group;
                })
                .then(group => {
                    if(req.body.skills){
                        var skills = [];
                        var skill = req.body.skills.split(',');
                        for(var i=0;i< skill.length; i++)
                            skills.push({ 'groupId' : group.id, 'skillId' : skill[i] });
                        return db.GroupSkill.bulkCreate(skills, { updateOnDuplicate: ["groupId", "skillId"] }).then(r => group);
                    }else return group;
                })
                .then(group => {
                    if(req.file && req.file.filename){
                        queue.create('create-group', {
                            groupId: group.id,
                            fileName: req.file.filename,
                            companyId: req.user.companyId
                        }).save();
                    }
                    return group;
                })
                .then(r => {
                    db.Group.update({name: req.body.name,  parentId:req.body.parentId }, {where: {id: req.params.id, companyId: req.user.companyId}})
                    .then(group => {
                        var allTasks = [], members = JSON.parse(req.body.members);
                        var managerId = null, deleteCount = 0;
                        for(var i=0;i<members.length;i++){
                            allTasks.push(new Promise((resolve, reject)=> {
                                var member = members[i];
                                if(member.mode == 'add') addedCount++;
                                if(member.mode == 'add' || member.mode == 'update'){
                                    addUpdateGroup(member, req.params.id)
                                    .then((dbMember) => {
                                        queue.create('assign-policy', { id : dbMember.id, modules : ['home','learner','course-analytics'] }).save();
                                        return db.GroupMember.findOne({ where: { groupId: req.params.id, memberId: dbMember.id } }).then(r => [r, dbMember])
                                    })
                                    .then(([isAvailable, dbMember]) => {
                                        if(!isAvailable)
                                            return db.GroupMember.create({ groupId: req.params.id, memberId: dbMember.id, isManager: (member.isManager === 'true' ? 1 : 0)  }).then(r => dbMember);
                                        else
                                            return db.GroupMember.update({ isManager: (member.isManager === 'true' ? 1 : 0)  }, {
                                                where: { id: isAvailable.id } 
                                            }).then(r => dbMember);
                                    })
                                    .then((dbMember) => {
                                        if(member.isManager === 'true'){
                                            managerId =  dbMember.id;
                                        if(dbMember)
                                            db.Group.update({managerId: managerId}, {where: {id: req.params.id, companyId: req.user.companyId} });
                                        } 
                                        resolve(true);
                                    })
                                    .catch(err => {
                                        reject(err);
                                    })
                                }else{
                                    db.Member.findOne({ where: { email: member.email }  })
                                    .then(t => {
                                        if(t) return db.GroupMember.destroy({ where: { groupId: req.params.id, memberId: t.id }});
                                        else throw new RequestError('Member doesn\'t Exists!');     
                                    })
                                    .then(r => {
                                        ++deleteCount;
                                        resolve(true);
                                    })
                                    .catch(err => {
                                        reject(err);
                                    })
                                } 
                            }))
                        }
                        return Promise.all(allTasks).then(r => deleteCount);
                    })
                    .then(deleteCount => {
                        return db.Group.findOne({ where: {id: req.params.id, companyId: req.user.companyId}, attributes: ['id', 'userCount']}).then(r => [r, deleteCount]);
                    })
                    .then(([r, deleteCount]) => {
                        return db.Group.update({userCount: (r.userCount)+addedCount-deleteCount}, {where: {id: req.params.id, companyId: req.user.companyId}, returning:true})
                    })
                    .then(r => {
                        res.status(200).json({'success': true});
                    })
                    .catch(err => {
                        console.log(err);
                        next(new RequestError('Error Updating Department!'));        
                    })
                })
                .catch(err => {
                    console.log(err);
                    next(new RequestError('Error Updating Department!'))        
                })
            }else throw new RequestError('Department doesn\'t exists!');
        }catch(err){
            console.log(err);
            next(new RequestError('Error updating group!'))  
        }       
    },

    async checkPermission(req, res, next){
        db.GroupMember.count({ include:[{ model:db.Group, where: { companyId: req.user.companyId} }] })
        .then(count => {
           return db.UserPackage.findOne({ where: { name: 'users', companyId: req.user.companyId } }).then(r => [r, count])
        })
        .then(([userPack, count]) => {
            var members = JSON.parse(req.body.members);
            var memberCount = 0;
            for(var i=0;i<members.length;i++){
                if(members[i].mode == 'add')
                    ++memberCount;
            }
            var total = parseInt(count) + (req.file && req.file.filename ? parseInt(req.body.csvLength) : parseInt(memberCount));
            if(req.user.User.trial == 1){
                if(total > 5)
                    throw new RequestError("5 Users are allowed in this plan, if add more user then upgrade your plan");
            }
            if(userPack){
                if(total > userPack.value)
                    throw new RequestError(userPack.value+ " Users are allowed in this plan, if add more user then upgrade your plan");
            }
            return true;
        })
        .then(r => {
            res.status(200).json({success: true});
        })
        .catch(err => {
            next(err);
        })
    },

    async saveAD(req, res, next){
        var data = req.body;
        // var data =  { 
        //     url: 'ldap://192.168.5.95',
        //     // baseDN: 'kratikal.com',
        //     username: 'Administrator',
        //     password: '#Telnet123$',
        //     savePassword: true
        // }
        try{
            if(CheckIsValidDomain(data.baseDN)){
                AD.connectAD(data)
                .then(result =>{
                    if(result == true){
                        return db.MetaData.findOne({ where:{ key:'AD', companyId:req.user.companyId } })
                    }else throw new RequestError("Please check your AD details");
                })
                .then(metadata =>{
                    if(data.savePassword){
                        console.log(data.password);
                        data.password = encrypt(data.password); 
                    }else data.password = '';
                    if(!metadata) return db.MetaData.create({ companyId: req.user.companyId, key:'AD', value:JSON.stringify(data) })
                    else return db.MetaData.update({value:JSON.stringify(data)}, { where:{ id: metadata.id} })
                })
                .then(metadata =>{
                    if(metadata) res.status(200).json({success: true});
                    else throw new RequestError("Error creating AD details.");
                })
                .catch(err =>{
                    next(err);
                })
            }else  throw new RequestError("Please enter a valid domain name.");
        }catch(err){
            next(err)
        }
    },

    async checkPassword(req, res, next){
        db.MetaData.findOne({ where: { companyId:req.user.companyId, key:'AD' } })
        .then(metadata =>{
            if(metadata){
                var metadata = JSON.parse(metadata.value);
                if(metadata.savePassword && metadata.password != ""){
                    metadata.password = decrypt(metadata.password);
                    if(CheckIsValidDomain(metadata.baseDN))
                        return AD.connectAD(metadata).then(r => [r, true])
                    else throw new RequestError("Please enter a valid domain name.", 403);
                }else{
                    if(req.body.password != ""){
                        metadata.password = decrypt(req.body.password);
                        return AD.connectAD(metadata).then(r => [r, true])
                    }else return [true, false];
                }  
            }else throw new RequestError('Please Add AD details from settings.', 404)
        })
        .then(([ad, check]) =>{
            if(ad == true){
                res.status(200).json({success: true, check:check });
            }else throw new RequestError("Please check your AD details", 403);
        })
        .catch(err =>{
            next(err);
        })
        .catch(err =>{
            next(err);
        })
    },

    async loginPassword(req, res, next){
        db.MetaData.findOne({ where: { companyId:req.user.companyId, key:'AD' } })
        .then(data =>{
            if(data){
                var metadata = JSON.parse(data.value);
                metadata.password = req.body.password;
                return AD.connectAD(metadata).then(r => [r, data, metadata]);
            }else throw new RequestError('AD details is not found.', 404);
        })
        .then(([ad, data, metadata]) =>{
            if(ad == true){
                metadata.password = encrypt(metadata.password);
                if(req.body.savePassword){
                    metadata.savePassword = req.body.savePassword;
                    return db.MetaData.update({value: JSON.stringify(metadata)}, { where:{ id: data.id } }).then(r => metadata)
                }else return metadata;
            }else throw new RequestError("Please check your AD details", 403);
        })
        .then(metadata =>{
            res.status(200).json({success: true, key:metadata.password});
        })
        .catch(err =>{
            next(err);
        })
    },

    async loadGroupUsers(req, res, next){
        db.MetaData.findOne({ where: { companyId:req.user.companyId, key:'AD' } })
        .then(metadata =>{
            if(metadata){
                var metadata = JSON.parse(metadata.value);
                if(CheckIsValidDomain(metadata.baseDN)){
                    var baseDomain = metadata.baseDN.split(".");
                    var data = {
                        url: (metadata.ssl == 'http' ? 'ldap://'+metadata.url: 'ldaps://'+metadata.url),
                        baseDN: 'dc='+baseDomain[0]+',dc='+baseDomain[1],
                        username: metadata.username,
                        password:(metadata.password ? decrypt(metadata.password) : decrypt(req.query.key)),
                        search:req.query.search
                    }
                    return AD.getGroups(data);
                }else  throw new RequestError("Please enter a valid domain.");
            }else throw new RequestError('AD details is not found.', 404)
        })
        .then(groups =>{
            if(groups.length > 0){
                res.status(200).json({success: true, groups:JSON.parse(groups)});
            }else res.status(200).json({success: true, groups:[]});
        })
        .catch(err =>{
            next(err);
        })
        // var data =  { 
        //     url: 'ldap://192.168.5.95',
        //     baseDN: 'kratikal.com',
        //     username: 'Administrator',
        //     password: '#Telnet123$',
        //     savePassword: true
        // }
    },

    async loadUsers(req, res, next){
        db.MetaData.findOne({ where: { companyId:req.user.companyId, key:'AD' } })
        .then(metadata =>{
            if(metadata){
                var metadata = JSON.parse(metadata.value);
                if(CheckIsValidDomain(metadata.baseDN)){
                    var baseDomain = metadata.baseDN.split(".");
                    var data = {
                        url: (metadata.ssl == 'http' ? 'ldap://'+metadata.url: 'ldaps://'+metadata.url),
                        baseDN: 'dc='+baseDomain[0]+',dc='+baseDomain[1],
                        username: metadata.username,
                        password:(metadata.password ? decrypt(metadata.password) : decrypt(req.query.key)),
                        search:req.query.search
                    }
                    return AD.getUsers(data);
                }else  throw new RequestError("Please enter a valid domain.");
            }else throw new RequestError('AD details is not found.', 404)
        })
        .then(users =>{
            if(users.length > 0) res.status(200).json({success: true, users:JSON.parse(users)});
            else res.status(200).json({success: true, users:[]});
        })
        .catch(err =>{
            next(err);
        })
    },

    async createUserGroups(req, res, next){
        db.MetaData.findOne({ where: { companyId:req.user.companyId, key:'AD' } })
        .then(metadata =>{
            if(metadata){
                var metadata = JSON.parse(metadata.value);
                metadata.password = (metadata.password ? decrypt(metadata.password) : decrypt(req.body.key));
                return AD.connectAD(metadata).then(r => [r, metadata]);
            }else throw new RequestError('AD details is not found.', 404)
        })
        .then(([ad, metadata]) =>{
            if(ad == true){
                var groups = req.body.userGroups;
                var OBJ = {};
                groups.forEach(grp => {
                    AD.getUsersForGroups(metadata, grp)
                    .then(userData =>{
                        console.log(grp)
                        OBJ[grp] = userData;
                        console.log(OBJ);
                    })
                });
                console.log(OBJ);
            }
            res.status(200).json({success: true, users:[]});
        })
        .catch(err =>{
            next(err);
        })
    },

    async hierarchyData(req,res,next){
        db.Group.findAll({attributes:['id','name','parentId'],where : {companyId:req.user.companyId}, order : [['id', 'DESC']] })
        .then(groups => {
            return res.status(200).json({groups: groups,success:true });
        })
        .catch(err => {
            next(err)
        })
    },
}; 
