import express from 'express';
import iamController from './iam.controller';
import { sanitize } from '../../../middleware/sanitizer';
import { jwtStrategy } from '../../../middleware/strategy';
import { userPermission } from '../../../middleware/userPermission';

export const iamRouter = express.Router();
iamRouter.route('/list').get(jwtStrategy,userPermission('policy-read'),iamController.index);
iamRouter.route('/add').post(jwtStrategy,userPermission('policy-write'),iamController.addPolicy);
iamRouter.route('/update').patch(jwtStrategy,userPermission('policy-write'),iamController.updatePolicy);
iamRouter.route('/show/:policyId').get(jwtStrategy,userPermission('policy-read'),iamController.showPolicy);
iamRouter.route('/delete/:policyId').get(jwtStrategy,userPermission('policy-delete'),iamController.policyDelete);
iamRouter.route('/count/:policyId').get(jwtStrategy,userPermission('policy-read'),iamController.policyUserCount);
iamRouter.route('/addUserRoles').post(sanitize(), jwtStrategy,userPermission('policy-write'),iamController.addUserRoles);
iamRouter.route('/userRoleList').get(jwtStrategy,userPermission('policy-read'),iamController.userRoleList);
iamRouter.route('/userRoleCount').get(jwtStrategy,userPermission('policy-read'),iamController.userRoleCount);

