import express from 'express';
import memberController from './member.controller';
import { jwtStrategy } from '../../../middleware/strategy';
import { sanitize } from '../../../middleware/sanitizer';
import { userPermission } from '../../../middleware/userPermission';

export const memberRouter = express.Router();
const rateLimit = require("express-rate-limit");
const apiLimiter = rateLimit({
    windowMs: 1 * 60 * 1000, // 15 minutes
    max: 50, 
    message: "Too many accounts created from this IP, please try again after an hour"
  });
memberRouter.route('/').get(sanitize(),apiLimiter, jwtStrategy,userPermission('department-list'),memberController.index);
memberRouter.route('/').post(sanitize(),apiLimiter, jwtStrategy,userPermission('learner-read'), memberController.storeUserCourseQuiz);
memberRouter.route('/:id').get(sanitize(),apiLimiter, jwtStrategy,userPermission('learner-read'),memberController.restartUserCourseQuiz);
memberRouter.route('/email/unsubscribe').get(sanitize(),apiLimiter, jwtStrategy,userPermission('learner-read'),memberController.emailUnsubscribe);
memberRouter.route('/save/emailUnsubscribe').post(sanitize(),apiLimiter, jwtStrategy,userPermission('learner-read'),memberController.saveEmailUnsubscribe);
memberRouter.route('/unsubscribe').post(sanitize(),apiLimiter,memberController.unsubscribe);
