import { db } from '../../../models';
import { validateEmail } from '../../../functions';
import mailer from '../../../mailer';

var validateFromAddress = (email) => {
    return new Promise((resolve, reject) =>{
        let tempParts = email.split('<');
        if(tempParts.length>1){
            email = tempParts[1];
        }
        tempParts = email.split('>');
        if(tempParts.length){
            email = tempParts[0];
        }
        let emailCheck = validateEmail(email);
        if(emailCheck)
            resolve(true);
        else 
            resolve(false);
    })  
}

export default {
    async index(req, res, next) {
        db.Sender.findOne({ attributes:['id', 'name', 'fromAddress', 'host', 'username', 'ssl', 'pool', 'limit', 'delay', 'maxMessages', 'maxConnections', 'connectionTimeout', 'requiretls'], where: { companyId: req.user.companyId }, order:[['id', 'DESC']] })
        .then(sender => {
            if(sender)  return res.status(200).json({success: true, sender: sender});
            else return res.status(200).json({sender: false});
        })
        .catch(err => {
            next(err);       
        })
    },
    
    async create(req, res, next) {
        var senderId = 0;
        var body  = req.body;
        body.companyId = req.user.companyId;
        validateFromAddress(body.fromAddress)
        .then(checkFromAddress =>{
            if(!checkFromAddress)  throw new RequestError('Invalid From Address');
            else return db.Sender.create(body);
        })
        .then(sender =>{
            return res.status(200).json({'success': true, senderId: sender.id});
        })
        .catch(err => {
            next(err);
        })
    },

    async update(req, res) {
        var body = req.body;
        validateFromAddress(body.fromAddress)
        .then(email => {
            if(!email) throw new RequestError('Invalid From Address');
            else return  db.Sender.findOne({where: {id: req.params.id, companyId: req.user.companyId}})
        })
        .then(sender => {
            if(!sender) throw new RequestError('SMTP Profile doesn\'t Exists');
            else{
                return db.Sender.update({
                    name: body.name,
                    host: body.host,
                    username: body.username,
                    password: body.password,
                    fromAddress: body.fromAddress,
                    ignore_cert_errors: body.ignore_cert_errors,
                    requiretls: body.requiretls,
                }, { where: { id: req.params.id, companyId: req.user.companyId }});
            }
        })
        .then(r => {
            return res.status(200).json({'success': true});
        })
        .catch(err => {
           next(err);
        })
    },

    async updateAdvanceSetting(req, res, next){
        db.Sender.findOne({ 
            where: { 
                id: req.params.id, 
                companyId: req.user.companyId
            }
        })
        .then(sender => {
            if(sender){
                return db.Sender.update({
                    ssl: req.body.ssl,
                    pool: req.body.pool,
                    delay: req.body.delay&&req.body.delay.length?req.body.delay:null,
                    limit: req.body.limit&&req.body.limit.length?req.body.limit:null,
                    maxMessages: req.body.maxMessage&&req.body.maxMessage.length?req.body.maxMessage:null,
                    maxConnections: req.body.maxConnection&&req.body.maxConnection.length?req.body.maxConnection:null,
                    connectionTimeout: req.body.connectionTimeout&&req.body.connectionTimeout.length?req.body.connectionTimeout:null,
                }, {where: {id: sender.id}});
            } else {
                throw new RequestError("invalid senderId recieved.");
            }
        })
        .then(sender => {
            return res.status(200).json({'success': true});
        })
        .catch(err =>{
            console.log(err);
            next(err)
        })
    },

    async testEmail(req, res, next){
        db.Sender.findOne({ where: {id: req.body.senderId} })
        .then(sender => {
            if(sender){
                req.body.host = sender.host;
                req.body.username = sender.username;
                req.body.password = sender.password;
                req.body.fromAddress = sender.fromAddress;
                req.body['check'] = true;
                req.body['requiretls'] = sender.requiretls;
                return mailer.sendTestMail(req.body);
            }else return mailer.sendTestMail(req.body);
        })
        .then(r => {
            res.status(200).json({success: true});
        })
        .catch(err => {
            console.log(err);
            next(err);       
        })
    },

    async fetch(req, res, next) {
        db.Header.findAll({ attributes: ['id', 'senderId', 'key', 'value'], where: { senderId: req.query.senderId  } })
        .then(headers => {
            if(headers)  return res.status(200).json({success: true, headers: headers});
            else throw new RequestError('SMTP Profile doesn\'t Exists');
        })
        .catch(err => {
            next(err);       
        })
    },

    async addHeader(req, res, next) {
        db.Sender.findOne({where: {id: req.body.senderId, companyId: req.user.companyId }})
        .then(sender => {
            if(sender) return db.Header.findOne({where: {senderId: req.body.senderId, key: req.body.key}})
            else throw new RequestError('SMTP Profile doesn\'t Exists');
        })
        .then(header => {
            if(header){
                return db.Header.update({ value: req.body.value }, {where: {id: header.id}})
            }else{
                return db.Header.create({
                    senderId: req.body.senderId,
                    key: req.body.key,
                    value: req.body.value
                })
            }
        })
        .then(header => {
            res.status(200).json({success: true});
        })
        .catch(err => {
           next(err);
        })
    },

    async delete(req, res, next) {
        if(req.params && req.params.id){
            db.Header.destroy({ where: { id: req.params.id }})
            .then(r => {
                if(r){
                    res.status(200).json({success: true});
                }else{
                    throw new RequestError('Error deleting header!');
                }
            })
            .catch(err => {
                next(err);      
            })
        }else next(new RequestError('Invalid data received!'));
    },
}; 
