import { db } from '../../../models';
import Campaign from "../../../krademy/campaign";
import { differentDateFormat, addedHis ,sanitize } from "../../../functions";
import { queue } from '../../../kue';


export default {
    createSimulation: async (req, res, next) => {
        if(req && req.body){
            var uaParser = require('ua-parser-js');
            var ip = req.headers["x-real-ip"] || req.ip;
            var detail = {};
            detail.user_agent = req.headers['user-agent'];
            detail.systemDetail = uaParser(req.headers['user-agent']);
            detail.ip = ip;
            var info = { info: detail};
            var metaData = JSON.stringify(info);
            var userId = req.user.User.id;
            var username = req.user.User.firstName+' '+req.user.User.lastName;
            var text = sanitize(req.body.name)+' campaign created by '+username;
            Campaign.create(req.user.companyId,{
                scheduledAt: (req.body.scheduledAt ? new Date(req.body.scheduledAt) : ''),
                name: sanitize(req.body.name),
                url: req.body.url,
                type: req.body.type,
                groups: req.body.groups,
                courses:( req.body.courses ? req.body.courses : null),
                quizzes: (req.body.quizzes ? req.body.quizzes : null),
                isManager: req.body.isManager,
                metaData: metaData,
                userId: userId,
                text: text
            })
            .then(campaignId => {
                res.status(200).json({success: true, campaignId: campaignId});
            })
            .catch(err => {
                next(new RequestError('Error creating Campaign!'));
            })
        }else throw new RequestError('Invalid Data received.!');
    },

    saveSettings: async (req, res, next) => {
        if(req && req.params && req.params.campaignId){
            if(req && req.body){
                db.Campaign.findOne({ where : { id: req.params.campaignId }})
                .then(campaign => {
                    if(campaign) return db.Simulation.findOne({ where : { campaignId : campaign.id } });
                    else throw new RequestError('Invalid campaign received.!');
                })
                .then(simulation => {
                    if(simulation){
                        if(req.body.campaignType == 'course')
                            return db.SimulationReminder.destroy({ where: { simulationId: simulation.id, campaignCourseId: req.body.id, executedAt : null } }).then(r => simulation);
                        else if(req.body.campaignType == 'quiz')
                            return db.SimulationReminder.destroy({ where: { simulationId: simulation.id, campaignQuizId: req.body.id, executedAt : null } }).then(r => simulation);
                    }else throw new RequestError('Invalid simulation received.!');
                })
                .then(simulation => {
                    if(simulation){
                        var dataReminders = [];
                        var reminders = req.body.reminders;
                        for(var i=0;i< reminders.length; i++){
                            if(req.body.campaignType == 'course')
                                dataReminders.push({ 'simulationId' : simulation.id, 'campaignCourseId' : req.body.id, 'type' : reminders[i].type, 'percent': reminders[i].percent, 'scheduledAt': reminders[i].scheduledAt });
                            else if(req.body.campaignType == 'quiz')
                                dataReminders.push({ 'simulationId' : simulation.id, 'campaignQuizId' : req.body.id, 'type' : reminders[i].type, 'percent': reminders[i].percent, 'scheduledAt': reminders[i].scheduledAt });
                        }
                        return db.SimulationReminder.bulkCreate(dataReminders, {returning: true} );
                    }else throw new RequestError('Invalid simulation received!');
                })
                .then(result => {
                    if(result)
                        res.status(200).json({ success: true });
                    else throw new RequestError('Error Updating Reminders!');
                })
                .catch(err => {
                    console.log(err);
                    next(err);
                })
            }else throw new RequestError('Invalid Data received.!');
        }else throw new RequestError('Invalid Campaign Id received.!');
    },

    dailyCron: async (req, res, next) => {
        var today = addedHis(differentDateFormat(5));
        db.SimulationReminder.findAll({ 
            attributes:['id', 'type', 'simulationId', 'campaignCourseId', 'percent', 'scheduledAt'], 
            include:[
                { model: db.Simulation, attributes:['campaignId'], required:true, include:[{ model:db.Campaign,  attributes:['companyId'],required:true }] }, 
                { model: db.CampaignCourse, attributes:['courseId'], required:true, include:[{ model: db.Course, attributes:['id'], required:true }] }
            ], 
            where: { executedAt: null, scheduledAt:today } 
        })
        .then(simulationReminders => {
            if(simulationReminders && simulationReminders.length > 0){
                var companies = {};
                for(var i = 0; i < simulationReminders.length; ++i){
                    var ad = {
                        'id': simulationReminders[i].id,
                        'simulationId': simulationReminders[i].simulationId, 
                        'courseId': (simulationReminders[i].CampaignCourse ? simulationReminders[i].CampaignCourse.Course.id : ''), 
                        'type': simulationReminders[i].type,  
                        'percent': simulationReminders[i].percent,
                        'scheduledAt': simulationReminders[i].scheduledAt
                    }
                    var all = {}, simulations = [], courses = [];
                    if(!companies[simulationReminders[i].Simulation.Campaign.companyId]){
                        companies[simulationReminders[i].Simulation.Campaign.companyId] = {};
                        if(simulationReminders[i].CampaignCourse && simulationReminders[i].CampaignCourse.Course){
                            if(!all[simulationReminders[i].simulationId+"-"+simulationReminders[i].CampaignCourse.Course.id])
                                all[simulationReminders[i].simulationId+"-"+simulationReminders[i].CampaignCourse.Course.id] = [];
                            all[simulationReminders[i].simulationId+"-"+simulationReminders[i].CampaignCourse.Course.id].push(ad);
                            if(courses.indexOf(simulationReminders[i].CampaignCourse.Course.id) == -1)
                                courses.push(simulationReminders[i].CampaignCourse.Course.id);
                            all['courses'] = courses;
                            if(simulations.indexOf(simulationReminders[i].simulationId) == -1)
                                simulations.push(simulationReminders[i].simulationId);
                            all['simulations'] = simulations;
                            companies[simulationReminders[i].Simulation.Campaign.companyId] = all;
                        }
                    }else{
                        var data = companies[simulationReminders[i].Simulation.Campaign.companyId];
                        if(!data[simulationReminders[i].simulationId+"-"+simulationReminders[i].CampaignCourse.Course.id])
                            data[simulationReminders[i].simulationId+"-"+simulationReminders[i].CampaignCourse.Course.id] = [];
                        data[simulationReminders[i].simulationId+"-"+simulationReminders[i].CampaignCourse.Course.id].push(ad);
                        if(data.courses.indexOf(simulationReminders[i].CampaignCourse.Course.id) == -1)
                            data.courses.push(simulationReminders[i].CampaignCourse.Course.id);
                        if(data.simulations.indexOf(simulationReminders[i].simulationId) == -1)
                            data.simulations.push(simulationReminders[i].simulationId);
                        companies[simulationReminders[i].Simulation.Campaign.companyId] = data;
                    }
                }
                // console.log(companies);
                for(var j = 0; j < Object.keys(companies).length; ++j){
                    queue.create('course-reminder-mails-cron-job', {
                        dataAll: Object.values(companies)[j],
                        companyId: Object.keys(companies)[j],
                        simulations: Object.values(companies)[j].simulations,
                        courses:Object.values(companies)[j].courses
                    }).removeOnComplete(true).save();
                }
                return true;
            }else return true;
        })
        .then(r => {
            console.log(today, '-----------------------------Cron Job is successfully done.------------------------------');
            if(r) res.status(200).json({ success: true });
            else throw new RequestError('Error Running Cron Job');
        })
        .catch(err => {
            console.log(err);
            next(err);
        })
    },
};