import { db } from '../../../models';
import { sanitize } from './../../../functions';

export default {
    async index(req, res, next) {
        var orderType = (req.query.type ? req.query.type : "DESC");
        db.Skill.findAll({ where: { companyId: req.user.companyId }, order: [ ['id', orderType] ]})
        .then(skills => {
            return res.status(200).json({skills: skills});
        })
        .catch(err => {
            next(err)
        })
    },

    async skillsByParent(req, res, next) {
        var orderType = (req.query.type ? req.query.type : "ASC");
        db.Skill.findAll({ where: { companyId: req.user.companyId }, order: [ ['id', orderType] ]})
        .then(skills => {
            return res.status(200).json({skills: skills});
        })
        .catch(err => {
            next(err)
        })
    },
    
    async create(req, res, next) {
        var uaParser = require('ua-parser-js');
        var ip = req.headers["x-real-ip"] || req.ip;
        var detail = {};
        detail.user_agent = req.headers['user-agent'];
        detail.systemDetail = uaParser(req.headers['user-agent']);
        detail.ip = ip;
        var info = { info: detail};
        var userId = req.user.User.id;
        var type = 'skill';
        var metaData = JSON.stringify(info);
        var companyId = req.user.companyId;
        var username = req.user.User.firstName+' '+req.user.User.lastName;
        var text = sanitize(req.body.name)+' skill created by '+username; 
        var body  = req.body;
        body.name =sanitize(req.body.name);
        body.companyId = req.user.companyId;
        db.Skill.findOne({ where: { name: body.name, companyId: req.user.companyId } })
        .then(skillCheck => {
            if(!skillCheck){
                db.RecentActivity.create({userId, companyId, type, text, metaData});
                return db.Skill.create(body);
            }else throw new RequestError('Skill with same name already exists!');
        })
        .then(skill => {
            if(skill)
                return res.status(200).json({'member': 'Skill is added successfully!', name:skill.name, diX342:skill.id, gif$5ds:skill.parentId});
            else throw new RequestError('Error creating Skill');
        })
        .catch(err => {
            next(err)
        })
    },

    async fetch(req, res, next) {
        if(req.params && req.params.id > 0){
            db.Skill.findOne({ where: { id:req.params.id, companyId: req.user.companyId } })
            .then(skill => {
                if(skill)
                    return res.status(200).json({skill: skill});
                else throw new RequestError('Skill doesn\'t exists!');
            })
            .catch(err => {
                next(err)
            })
        }else next(new RequestError('Invalid data received!'));
    },

    async update(req, res, next) {
        var body = req.body;
        if(Object.keys(body).length > 0){
            db.Skill.findOne({ where: { id:body.skillId, companyId: req.user.companyId } })
            .then(checkSkillId => {
                if(checkSkillId){
                    return db.Skill.findOne({where: { name: sanitize(body.name), companyId: req.user.companyId, id: { $not: body.skillId } } });
                }else throw new RequestError('Skill doesn\'t exists!');
            })
            .then(checkSkill => {
                if(!checkSkill){
                    return db.Skill.update({ name: sanitize(body.name) }, {where: {id: body.skillId, companyId: req.user.companyId} });
                }else throw new RequestError('Skill with same name already exists!');
            })
            .then(skill => {
                if(skill)
                    return res.status(200).json({skill: 'Skill is successfully updated!'});
                else throw new RequestError('Error updating skill!');
            })
            .catch(err => {
                next(err)
            })
        }else next(new RequestError('Invalid data received!'));
    },

    async fetchAll(req, res, next) {
        let limit = 10;
        let offset = 0;
        let page = 1;
        if(req.query.limit != undefined){
            limit = parseInt(req.query.limit);
        }
        if (req.query.page) {
            page = req.query.page;
            if (page < 1)
                page = 1;
        }
        if(req.params && req.params.id){
            db.Skill.count({ where: { parentId:req.params.id, companyId: req.user.companyId }})
            .then(skillCount => {
                let count = skillCount;
                let pages = Math.ceil(count / limit);
                offset = limit * (page - 1);
                return db.Skill.findAll({ 
                    where: { parentId:req.params.id, companyId: req.user.companyId }, 
                    order: [ ['id', 'DESC'] ],
                    limit: limit,
                    offset: offset
                }).then(r => [r, pages, count])
            })
            .then(([skills, pages, count]) => {
                if(skills)
                    return res.status(200).json({skills: skills, 'count': count, 'pages': pages });
                else throw new RequestError('Skill doesn\'t exists!');
            })
            .catch(err => {
                next(err)
            })
        }else next(new RequestError('Invalid data received!'));
    },

    async delete(req, res, next) {
        if(req.params && req.params.id){
            db.Skill.findOne({ where: { id:req.params.id, companyId: req.user.companyId } })
            .then(skills => {
                if(skills)
                    return db.Skill.destroy({ where: { id: req.params.id, companyId: req.user.companyId }});    
                else throw new RequestError('Skill doesn\'t exists!');
            })
            .then(r => {
                return db.Skill.update({ parentId: 0 }, {where: {parentId: req.params.id} });
            })
            .then(skill => {
                if(skill)
                    return res.status(200).json({skill: 'Poof! Your skill has been deleted!'});
                else throw new RequestError('Error Deleting Skill!');
            })
            .catch(err => {
                next(err)
            })
        }else next(new RequestError('Invalid data received!'));
    },
}; 
