import { db } from '../../../models';
import mailer from '../../../mailer';

export default {
    async index(req, res, next) {
        let limit = 10;
        let offset = 0;
        let page = 1;
        if (req.query.limit != undefined) {
            limit = parseInt(req.query.limit);
        }
        if (req.query.page) {
            page = req.query.page;
            if (page < 1)
                page = 1;
        }
        db.Templates.count({ where: { type: {$notIn:['certificate', 'assessment_certificate'] } } })
        .then((count) => {
            let pages = Math.ceil(count / limit);
            offset = limit * (page - 1);
            return db.Templates.findAll({
                attributes: ['id', 'name', 'type', 'subject'],
                where: { type: {$notIn:['certificate', 'assessment_certificate'] } },
                limit: limit,
                offset: offset,
            }).then(r => [r, count, pages]);
        })
        .then(([templates, count, pages]) => {
            return db.MetaData.findAll({
                attributes: ['key', 'value'],
                where:{ companyId: req.user.companyId, key: { like: '\\_\\_%' }}
            }).then(r => [r, templates, count, pages])
        })
        .then(([metadata, templates, count, pages]) => {
            return res.status(200).json({ templates: templates,'count': count, 'pages': pages, metadata });
        })
        .catch(err => {
            console.log(err);
            next();
        })
    },

    async save(req, res, next) {
        let data = req.body;
        if(req.file) data['filename'] = req.file.filename;
        db.Templates.findOne({ where: { id:data.templateId }  })
        .then(template => {
            if(template){
               return db.MetaData.findOne({ where:{ companyId: req.user.companyId, key:template.name } }).then(r =>[r, template])
            }else throw new RequestError('Error fetching template.');
        })
        .then(([metadata, template]) => {
            if(!metadata){
                var dataContent = {
                    key: template.name,
                    value: JSON.stringify(data),
                    companyId: req.user.companyId,
                }
                return db.MetaData.create(dataContent);
            }else{
                let updateData = JSON.parse(metadata.value);
                updateData.subject = data.subject;
                updateData.type = data.type;
                if(data.type == 'text'){
                    updateData.upper_content = data.upper_content;
                    updateData.middle_content = data.middle_content;
                    updateData.lower_content = data.lower_content;
                }else{
                    if(req.file){
                        updateData.originalName = req.file.originalname;
                        updateData.filename = req.file.filename;
                    }
                } 
                return db.MetaData.update({ value: JSON.stringify(updateData) }, { where:{ id:metadata.id } }).then(r => metadata)
            } 
        })
        .then(result => {
            return res.status(200).json({ success: true, meta:result.id });
        })
        .catch(err => {
            console.log(err);
            next();
        })
    },

    async fetch(req, res, next) {
        db.Templates.findOne({ where: { id:req.query.tId }  })
        .then(template => {
            if(template){
                return db.MetaData.findOne({ where:{ companyId:req.user.companyId, key: template.name}, attributes:['value', 'id'] })
            }else throw new RequestError('Error fetching template.');
        })
        .then(metaData => {
            return db.Company.findOne({ where: { id:req.user.companyId },attributes:['name', 'logo'] }).then(r =>[r, metaData])
        })
        .then(([company, metaData]) => {
            return res.status(200).json({ success: true, metaData, company  });
        })
        .catch(err => {
            console.log(err);
            next();
        })
    },

    async change(req, res, next) {
        db.Templates.findOne({ where: { id:req.query.tId }  })
        .then(template => {
            if(template){
                return db.MetaData.findOne({ where:{ companyId:req.user.companyId, key: template.name}, attributes:['id', 'value'] })
            }else throw new RequestError('Error fetching template.');
        })
        .then(metaData => {
            if(metaData){
                var data = JSON.parse(metaData.value);
                data.enable = (req.query.type == 'd' ? false : true);
                return db.MetaData.update({ value: JSON.stringify(data) }, { where:{ id:metaData.id } });
            }else throw new RequestError('Error changing template.');
        })
        .then(result => {
            return res.status(200).json({ success: true  });
        })
        .catch(err => {
            console.log(err);
            next();
        })
    },

    async changeTextOrImage(req, res, next) {
        db.MetaData.findOne({ where:{ companyId:req.user.companyId, id:req.query.id}, attributes:['id', 'value'] })
        .then(metaData => {
            if(metaData){
                var data = JSON.parse(metaData.value);
                data.type = req.query.type;
                return db.MetaData.update({ value: JSON.stringify(data) }, { where:{ id:metaData.id } });
            }else throw new RequestError('Error changing template.');
        })
        .then(result => {
            return res.status(200).json({ success: true  });
        })
        .catch(err => {
            console.log(err);
            next();
        })
    },

};