import express from 'express';
import templateController from './template.controller';
import { sanitize } from '../../../middleware/sanitizer';
import { jwtStrategy } from '../../../middleware/strategy';
import { userPermission } from '../../../middleware/userPermission';
import multer from 'multer';
import path from 'path';
import config from '../../../config';

var attachmentDir = path.join(config.storage.folder, config.storage.template_custom)
var storage = multer.diskStorage({
    destination: function (req, file, cb) {
        if (file.fieldname === "logo") cb(null, attachmentDir)
    },
    filename: function (req, file, cb) {
        var originalname = file.originalname;
        if (file.mimetype.includes('image')) originalname = file.originalname + '.png';
        cb(null, Date.now() + path.extname(originalname))
    }
})
var uploadAttachment = multer({ storage: storage, limits: { fileSize: 1024 * 1024 * 1024 },

});

export const templateRouter = express.Router();
templateRouter.route('/').get(sanitize(), jwtStrategy, userPermission('template-list'), templateController.index);
templateRouter.route('/').post(sanitize(), jwtStrategy, userPermission('template-write'), uploadAttachment.single('logo'),templateController.save);
templateRouter.route('/edit').get(sanitize(), jwtStrategy, userPermission('template-read') ,templateController.fetch);
templateRouter.route('/change').get(sanitize(), jwtStrategy, userPermission('template-write') ,templateController.change);
templateRouter.route('/change-text-image').get(sanitize(), jwtStrategy, userPermission('template-write') ,templateController.changeTextOrImage);
