import express from 'express';
import logger from 'morgan';
import path from 'path';
import passport from 'passport';
import session from 'express-session';
import bodyParser from 'body-parser';
import exphbs from 'express-handlebars';
import cookieParser from 'cookie-parser';
import expressSanitizer from 'express-sanitizer';
import helmet from 'helmet';
import rfs from 'rotating-file-stream';
import fs  from 'fs';

var foldersArrry = [ 'company', 'course', 'quiz', 'template' ];

export default {
    setup: (config, defaultLayout) => {
        const app = express();

        // var accessLogStream = rfs('access.log', {
        //     interval: '1d',
        //     path: path.join(__dirname, '..', 'log')
        // })
        
        // app.use(logger(config.app.log, { stream: accessLogStream }));

        app.use(bodyParser.urlencoded({ extended: true }));
        app.use(bodyParser.json({limit: '50mb'}));

        app.use(cookieParser(config.app.secret));
        app.use(session({ secret: config.app.secret ,resave: true, saveUninitialized:true}));
        app.use(passport.initialize());
        app.use(passport.session());
        app.use(expressSanitizer());
        app.use((req, res, next) => {
            res.locals.currentURL = req.path;
            if(req.url.startsWith('/static/')){
                let getUrl = req.url.split('/static/');
                let againSplit= getUrl[1].split('/');
                if(foldersArrry.includes(againSplit[0]) && fs.existsSync(config.storage.folder+"/"+getUrl[1])){
                    res.sendFile(config.storage.folder+"/"+getUrl[1]);
                }else next();
            }else next();
            // if(req.url.indexOf("/content/") != -1 && req.url.indexOf("/content/thumbnail") == -1){  
            //     if(fs.existsSync("/thirty/content/"+req.url.split("/content/")[1]))
            //         res.sendFile("/thirty/content/"+req.url.split("/content/")[1]);
            //     else if(fs.existsSync(config.storage.folder+"/content/"+req.url.split("/content/")[1]))
            //         res.sendFile(config.storage.folder+"/content/"+req.url.split("/content/")[1]);
            //     else next();
            // }else next();
        });
        app.use("/static", express.static(path.join(__dirname, 'public'), { maxAge: 3600000 }));
        app.use(helmet());
        app.use(helmet.hsts({
            maxAge: 0
        }))
        app.use(function(req, res, next){
            res.header('Cache-Control', 'no-cache, no-store, private, must-revalidate');
            res.header('Expires', -1);
            res.header('Pragma', 'no-cache');
            next();
        });
        app.set('views', path.join(__dirname, 'web', 'resources', 'views', 'layouts'));
        app.engine('hbs', exphbs({
            extname: '.hbs',
            defaultLayout: defaultLayout,
            partialsDir: path.join(__dirname, 'web', 'resources', 'views', 'partials'),
            layoutsDir: path.join(__dirname, 'web', 'resources', 'views', 'layouts'),
            helpers: {
                block: function (name) {
                    var blocks  = this._blocks,
                        content = blocks && blocks[name];

                    return content ? content.join('\n') : null;
                },
                contentFor: function (name, options) {
                    var blocks = this._blocks || (this._blocks = {}),
                        block  = blocks[name] || (blocks[name] = []);

                    block.push(options.fn(this));
                },
                if: function (name, options) {
                    if(name) {
                        return options.fn(this);
                    } else {
                        return options.inverse(this);
                    }
                },
                if_eq: function(a, b, opts) {
                    if (a.indexOf(b) !=-1) {
                        return opts.fn(this);
                    } else {
                        return opts.inverse(this);
                    }
                }
        }
        }));
        app.set('view engine', '.hbs');

        Number.prototype.pad = function (size) {
            var s = String(this);
            while (s.length < (size || 2)) { s = "0" + s; }
            return s;
        }
        
        return app;
    }
}