import kue from 'kue';
// import nodemailer from 'nodemailer';
import { db } from './models';
import config from './config';
import fs from 'fs';
import stringifyCSV from 'csv-stringify';
import csvParser from 'csv-parser';
import { makenumerickey } from './functions';
import scheduler from './scheduler';
import path from 'path';
import { fork } from 'child_process';
import mailer from './mailer';
import { promisify } from 'util';
import Blowfish from 'egoroof-blowfish';
import Simulation from "./krademy/simulation";
import rd from 'redis';
import nodemailer from 'nodemailer';
import { differentDateFormat } from './functions'

const client = rd.createClient();
const redis = {
    get: promisify(client.get).bind(client),
    set: promisify(client.set).bind(client),
    delete: promisify(client.del).bind(client),
}

var getTemplateType = {
    course: '__Send_Course__',
    quiz: '__Send_Assessment__',
    completion: '__Course_Completion__',
    reminder: '__Course_Reminder__',
    quiz_fail: '__Course_Quiz_Fail_Reminder__'
}


var cpuu = require('cputilization');
var sampler = cpuu({ interval: 1000 });
var groupsMembers = {};

function makeid() {
    var result = '', characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789', charactersLength = 62;
    for (var i = 0; i < 30; i++) { result += characters.charAt(Math.floor(Math.random() * charactersLength)) }
    return result;
}

function sendErrorMessage(sessions, type) {
    let session_obj = {}; let sessionId_arr = [];
    return new Promise((resolve, reject) => {
        let CampaignCoursesorQuizzes = (type == 'course' ? sessions[0].Simulation.Campaign.CampaignCourses : sessions[0].Simulation.Campaign.CampaignQuizzes);
        let arr = CampaignCoursesorQuizzes.map(ele => {
            if (type == 'course') return { id: ele.Course.id, name: ele.Course.name, image: ele.Course.logo }
            else return { id: ele.Quiz.id, name: ele.Quiz.name, image: ele.Quiz.logo };
        });
        sessions.forEach((element, i) => {
            if (Object.keys(session_obj).indexOf(element.email) == -1) {
                session_obj[element.email] = {
                    memberId: element.memberId,
                    sessionId: element.id,
                    sessionKey: element.sessionKey,
                    firstName: element.firstName,
                    lastName: element.lastName,
                    email: element.email,
                    subscribe: (type == 'course' ? element.Member.courseMails : element.Member.assessmentMails),
                    code: element.Member.code,
                    courseOrQuiz: arr,
                    type: type
                }
            }
            sessionId_arr.push(element.id)
        })
        resolve();
    })
        .then(r => {
            return db.Session.update({ status: 'Pending' }, { where: { id: { in: sessionId_arr } } })
        })
        .then(result => {
            return session_obj;
        }).catch(err => {
            reject(err)
        });
}

function storeMemberInFile(sessionHash, groupId, simulationId, reminderId, member) {
    let filePath = appRoot + '/public/managermails/' + `${sessionHash}-${groupId}-${simulationId}-${reminderId}.csv`;
    if (!fs.existsSync(filePath)) {
        fs.writeFileSync(filePath, `First Name, Last Name, Email, Percentage\n`);
    }
    fs.appendFileSync(filePath, `${member.firstName}, ${(member.lastName ? member.lastName : '-')}, ${member.email}, ${member.percentage + '%'}\n`);
}

function sendReminderToManagers(sessionHash) {
    let filePath = appRoot + '/public/managermails/';
    fs.readdirSync(filePath).forEach(async file => {
        if (file.startsWith(sessionHash) && file.endsWith(".csv") && file.split("-").length == 4) {
            let managerId = file.split("-")[1];
            queue.create('send-manager', {
                managerId, file: filePath + file, fileName: file
            }).removeOnComplete(true).save()
        }
    })
}

const bf = new Blowfish(toString(config.app.secret), Blowfish.MODE.ECB, Blowfish.PADDING.NULL);
const fromHexString = hexString => new Uint8Array(hexString.match(/.{1,2}/g).map(byte => parseInt(byte, 16)));

var dcryptPassword = (password) => {
    return new Promise((resolve, reject) => {
        if (password && password.length) resolve(bf.decode(fromHexString(password)));
        else resolve(false);
    })
}

export var queue = kue.createQueue({
    prefix: 'q1',
    redis: {
        host: config.redis.host,
        port: config.redis.port,
        auth: config.redis.password
    }
});

var loadAvailable = false;
sampler.on('sample', function (sample) {
    loadAvailable = sample.percentageBusy() < 0.2
    // console.log(sample.percentageBusy());
});

var getMemberEmail = (members) => {
    let memberEmail = [];
    for (let i = 0; members && i < members.length; i++) {
        if (memberEmail.indexOf(members[i].email) == -1)
            memberEmail.push(members[i].email);
    }
    return memberEmail;
}

var getCourseData = members => {
    let memberCourse = [];
    members.forEach(ele => {
        if (members.indexOf(ele.courseId) == -1)
            memberCourse.push(ele.courseId)
    })
    return memberCourse;
}

var getRemainderData = members => {
    let memberRemainder = [];
    members.forEach(ele => {
        if (members.indexOf(ele.remainderId) == -1)
            memberRemainder.push(ele.remainderId)
    })
    return memberRemainder;
}


function createErrorCsv(items, groupId) {
    return new Promise((resolve, reject) => {
        stringifyCSV(items, function (err, output) {
            if (err) reject(err);
            let fileName = Date.now();
            fs.writeFile(appRoot + '/public/groupscsv/' + fileName + '.csv', output, function (err) {
                if (err) { reject(err); throw err; };
                db.Group.update({
                    errorFile: fileName + '.csv'
                }, { where: { id: groupId } })
                resolve(fileName + '.csv');
            });
        })
    })
}

var readFile = (memberData) => {
    return new Promise((resolve, reject) => {
        var batching = config.mail.batching, memberObject = {}, count = 1, errorMembers = [];
        errorMembers.push(['firstName', 'lastName', 'email', 'phone', 'designation', 'isManager', 'error']);
        if (memberData.fileName) {
            fs.createReadStream(appRoot + '/public/groupscsv/' + memberData.fileName).pipe(csvParser())
                .on('data', function (data) {
                    try {
                        if (data) {
                            if (!memberObject[count]) memberObject[count] = [];
                            data = JSON.parse(JSON.stringify(data));
                            if (data && data.email) {
                                data.firstName = data.firstName ? data.firstName.trim() : null;
                                data.lastName = data.lastName ? data.lastName.trim() : null;
                                data.email = data.email ? (data.email.trim()).toLowerCase() : null;
                                data.designation = data.designation ? data.designation.trim() : null;
                                data.phone = data.phone ? data.phone.trim() : null;
                                data.isManager = data.isManager ? data.isManager.trim() : null;
                            }
                            if (data.firstName && data.firstName.length && data.email && data.email.length) {
                                memberObject[count].push(data);
                            } else {
                                var error = '';
                                // if(!domainName[(data.email).split('@')[1]]) error = 'Domain not approved';
                                errorMembers.push([
                                    data.firstName ? data.firstName : '',
                                    data.lastName ? data.lastName : '',
                                    data.email ? data.email : '',
                                    data.phone ? data.phone : '',
                                    data.designation ? data.designation : '',
                                    data.isManager ? data.isManager : '',
                                    error
                                ])
                            }
                            if (memberObject[count].length % batching == 0) {
                                queue.create('insert-member', {
                                    groupId: memberData.groupId,
                                    companyId: memberData.companyId,
                                    members: memberObject[count],
                                    complete: false
                                }).delay(count * 1000).priority('high').save();
                                count = count + 1;
                            }
                        } else {
                            console.log('Members not found in File');
                        }
                    }
                    catch (err) {
                        reject(err);
                        console.log(err);
                    }
                })
                .on('end', function () {
                    console.log('File read complete');
                    queue.create('insert-member', {
                        groupId: memberData.groupId,
                        companyId: memberData.companyId,
                        members: memberObject[count],
                        complete: true
                    }).save();
                    if (errorMembers.length > 1) createErrorCsv(errorMembers, memberData.groupId);
                    resolve(true);
                });
        } else {
            resolve(true);
        }
    })
}

var simulationBatching = (resultCount, simulationId, campaignId, quizzes, courses, type, companyId) => {
    db.Campaign.findOne({ where: { id: campaignId, companyId: companyId }, paranoid: false })
        .then(campaign => {
            if (campaign) {
                return db.Sender.findOne({ where: { companyId: companyId }, order: [['id', 'DESC']], include: [{ model: db.Company, attributes: ['smtpStatus', 'skipLogin'], required: true }] })
            } else throw new RequestError('Campaign Doesn\'t Exists')
        })
        .then(sender => {
            var delay = config.mail.delay, batchingRange = config.mail.batching;
            if (sender && sender.Company.smtpStatus == 'custom') {
                if (sender.delay && sender.delay != null)
                    delay = sender.delay;
                if (sender.limit && sender.limit != null)
                    batchingRange = sender.limit;
            }
            var miliseconds = 0;
            for (var i = 0; i < Math.ceil(resultCount / batchingRange); i++) {
                let offset = i * batchingRange;
                miliseconds = delay * i;
                queue.create('simulation-batch', {
                    simulationId: simulationId,
                    campaignId: campaignId,
                    courses: (courses ? courses : null),
                    quizzes: (quizzes ? quizzes : null),
                    offset: offset,
                    limit: batchingRange,
                    resultCount: resultCount,
                    type: type,
                    companyId: companyId
                }).delay(miliseconds).save();
            };
            return true;
        })
        .catch(err => {
            if (simulationId) {
                db.CampaignError.create({
                    campaignId: campaignId,
                    error: JSON.stringify(err)
                });
            }
            console.log('error - ' + JSON.stringify(err));
        })


}

export default {
    init: () => {
        queue.process('create-group', function (job, done) {
            console.log('Create group call');
            console.log(job.data);
            readFile(job.data)
                .then(r => {
                    return db.Group.update({
                        status: 'completed'
                    }, { where: { id: job.data.groupId, companyId: job.data.companyId } });
                })
                .then(r => {
                    done();
                })
                .catch(err => {
                    db.Group.update({
                        status: 'failed'
                    }, { where: { id: job.data.groupId, companyId: job.data.companyId } }).catch(err => {
                        console.log(err);
                    }).finally(r => {
                        done(new Error('Group creation failed - Queue'));
                    })
                })
        });

        queue.on('job failed', function (id, result) {
            if (result == 'system-load') {
                setTimeout(() => {
                    kue.Job.get(id, function (err, job) {
                        job.state('inactive').save();
                    });
                }, 1000);
            }
        });


        queue.process('send-manager', async (job, done) => {
            try {
                let manager = await redis.get(`manager-${job.data.managerId}`);
                let mailData = await redis.get(job.data.fileName.split(".")[0])
                mailer.sendMailToManagerForRemainingCourseReminder(JSON.parse(mailData), JSON.parse(manager), job.data.file);
                // await redis.delete(`manager-${job.data.managerId}`)
                // await redis.delete(job.data.fileName.split(".")[0])
                done();
            } catch (err) {
                console.log(err);
                done(err);
            }
        });

        queue.process('insert-member', (job, done) => {
            var members = job.data.members, groupId = job.data.groupId;
            if (!members && members.length == 0) return done(new Error("No members provided - Queue"));
            new Promise((resolve, reject) => {
                if (job.data.complete) {
                    return db.Group.update({
                        status: 'completed',
                        userCount: members.length
                    }, { where: { id: job.data.groupId, companyId: job.data.companyId } })
                        .then(r => { resolve(r) }).catch(err => { reject(err) });
                } else { resolve(true) }
            })
                .then(r => {
                    var memberEmail = getMemberEmail(members);
                    return db.Member.findAll({
                        attributes: ['id', 'email'],
                        distinct: true,
                        group: ['email'],
                        where: { email: { $in: memberEmail } }
                    })
                })
                .then(member => {
                    var insertedMemberEmails = {}, insertedMemberIds = [];
                    var addMemberData = [];
                    if (member && member.length) {
                        queue.create('update-member', {
                            member: member,
                            email: members
                        }).save();
                        for (var i = 0; i < member.length; i++) {
                            insertedMemberEmails[(member[i].email).toLowerCase()] = (member[i].email).toLowerCase();
                            insertedMemberIds.push(member[i].id)
                        }
                        for (var i = 0; i < members.length; i++) {
                            if (!insertedMemberEmails[(members[i].email).toLowerCase()]) {
                                members[i]['code'] = makeid();
                                addMemberData.push(members[i]);
                            }
                        }
                        return [addMemberData, insertedMemberIds];
                    } else {
                        return [members, insertedMemberIds];
                    }
                })
                .then(([members, memberIds]) => {
                    if (members && members.length) {
                        return db.Member.bulkCreate(members, { returning: true }).then(r => [r, memberIds]);
                    } else {
                        return [false, memberIds];
                    }
                })
                .then(([mem, memberIds]) => {
                    return db.GroupMember.findAll({
                        where: {
                            groupId: groupId,
                            memberId: { $in: memberIds }
                        }
                    }).then(r => [mem, memberIds, r]);
                })
                .then(([mem, memberIds, groupMember]) => {
                    var groupMemberIds = {}, groupMemberData = [];
                    for (var i = 0; i < groupMember.length; i++) {
                        groupMemberIds[groupMember[i].memberId] = groupMember[i].memberId;
                    }
                    for (var i = 0; i < memberIds.length; i++) {
                        if (!groupMemberIds[memberIds[i]]) {
                            groupMemberData.push({
                                groupId: groupId,
                                memberId: memberIds[i],
                            })
                        }
                    }
                    return [mem, groupMemberData];
                })
                .then(([mem, groupMemberData]) => {
                    if (mem && mem.length) {
                        for (var i = 0; i < mem.length; i++) {
                            groupMemberData.push({
                                groupId: groupId,
                                memberId: mem[i].id
                            });
                            queue.create('assign-policy', { id: mem[i].id, modules: ['home', 'learner', 'course-analytics'] }).save();
                        }
                    }
                    return groupMemberData;
                })
                .then(groupMemberData => {
                    if (groupMemberData && groupMemberData.length) {
                        return db.GroupMember.bulkCreate(groupMemberData, { returning: true })
                    } else return true;
                })
                .then(r => {
                    if (r)
                        return db.GroupMember.findAll({ where: { groupId: groupId } })
                })
                .then(groupMem => {
                    if (groupMem) {
                        var memberEmail, alreadyManager = true;
                        if (groupMem && groupMem.length) {
                            for (var i = 0; i < groupMem.length; i++) {
                                if (groupMem[i].isManager == 1)
                                    alreadyManager = false;
                            }
                        }
                        if (alreadyManager) {
                            var count = 0;
                            for (var i = 0; i < members.length; i++) {
                                var yesno = members[i].isManager;
                                if (yesno && yesno.toLowerCase() === 'yes' && count == 0) {
                                    memberEmail = members[i].email;
                                    count = 1;
                                }
                            }
                        }
                        if (memberEmail)
                            return db.Member.findOne({ where: { email: memberEmail } }).then(r => [r, groupMem])
                        else return [false, groupMem];
                    }
                })
                .then(([manager, groupMem]) => {
                    if (manager) {
                        return db.GroupMember.update({ isManager: 1 }, { where: { memberId: manager.id, groupId: groupId } }).then(r => [manager, groupMem])
                    } else return [manager, groupMem];
                })
                .then(([manager, groupMem]) => {
                    var dataOBJ = {
                        status: 'completed',
                        userCount: groupMem.length,
                    }
                    if (manager)
                        dataOBJ['managerId'] = manager.id;
                    return db.Group.update(dataOBJ, { where: { id: job.data.groupId, companyId: job.data.companyId } });
                })
                .then(r => {
                    done();
                })
                .catch(err => {
                    console.log(err);
                    db.Group.update({
                        status: 'failed'
                    }, { where: { id: job.data.groupId, companyId: job.data.companyId } }).catch(err => {
                        console.log(err);
                    }).finally(r => {
                        done(new Error('Error inserting member - Queue'));
                    })
                })
        })

        queue.process('update-member', (job, done) => {
            var member = job.data.member, csvMember = job.data.email;
            var memberData = {};
            for (var i = 0; i < csvMember.length; i++) {
                memberData[csvMember[i].email] = csvMember[i];
            }
            member.reduce((p, data) => {
                return db.Member.update({
                    firstName: memberData[data.email.toLowerCase()] ? memberData[data.email.toLowerCase()].firstName : data.fileName,
                    lastName: memberData[data.email.toLowerCase()] ? memberData[data.email.toLowerCase()].lastName : data.lastName,
                    phone: memberData[data.email.toLowerCase()] ? memberData[data.email.toLowerCase()].phone : data.phone,
                    designation: memberData[data.email.toLowerCase()] ? memberData[data.email.toLowerCase()].designation : data.designation,
                }, {
                    where: { id: data.id }
                })
            }, Promise.resolve(
                done()
            ))
                .catch(err => {
                    console.log(err)
                    return done(new Error('Update Member Failed - Queue'));
                });
        })

        queue.process('assign-policy', function (job, done) {
            db.Member.findOne({ where: { id: job.data.id } })
                .then(member => {
                    if (member) return db.MetaAuth.findOne({ where: { email: member.email } }).then(r => [r, member])
                    else throw new RequestError('kue error : invalid email . ');
                })
                .then(([meta, member]) => {
                    var key = makenumerickey();
                    // var passwordHash = bcrypt.hashSync(key);
                    if (meta) return db.MetaAuth.update({ memberId: member.id }, { where: { id: meta.id } })
                    else return db.MetaAuth.create({ email: member.email, memberId: member.id, valid: 1, isNewUser: 1 })
                })
                // .then(([r,member])=>{console.log(JSON.stringify(r))
                //     return db.UserPolicy.destroy({ where: { type: "member", memberId: member.id } }).then(r => [r, member]);
                // })
                // .then(([r, member]) => {
                //     return job.data.modules.reduce((p1, a1) => {
                //         return db.Policy.findAll({ where: { name: { like: '%' + a1 + '%' } } })
                //             .then(r => {
                //                 var id = [];
                //                 r.reduce((p2, a2) => {
                //                     id.push({ type: 'member', userId: null, memberId: member.id, policyId: a2.id })
                //                 }, Promise.resolve());
                //                 return db.UserPolicy.bulkCreate(id)
                //             })
                //     }, Promise.resolve())
                // })
                .then(r => {
                    done();
                })
                .catch(err => {
                    console.log(err);
                    done(new Error('Assign Policy Failed - Queue'));
                })
        });

        queue.process('simulation', function (job, done) {
            db.Simulation.update({ executedAt: new Date() }, { where: { id: job.data.simulationId }, returning: true, plain: true })
                .then(data => {
                    return db.Session.count({ attributes: ['id', 'simulationId'], where: { 'simulationId': job.data.simulationId } }).then(r => [r, job.data.simulationId, job.data.campaignId])
                })
                .then(([resultCount, simulationId, campaignId]) => {
                    return simulationBatching(resultCount, simulationId, campaignId, job.data.quizzes, job.data.courses, job.data.type, job.data.companyId);
                })
                .then(r => {
                    return done();
                })
                .catch(err => {
                    console.log(JSON.stringify(err));
                    return done(new Error('Simulation Failed - Queue'));
                });
        });

        queue.process('simulation-batch', (job, done) => {
            try {
                return new Promise((resolve, reject) => {
                    console.log('Start Child Process.....' + job.data.offset)
                    const process = fork(path.join(__dirname, 'krademy/simulation.js'));
                    process.send({
                        simulationId: job.data.simulationId,
                        campaignId: job.data.campaignId,
                        quizzes: job.data.quizzes,
                        courses: job.data.courses,
                        offset: job.data.offset,
                        limit: job.data.limit,
                        type: job.data.type,
                        companyId: job.data.companyId,
                        resultCount: job.data.resultCount
                    })
                    process.on('message', async (result) => {
                        console.log('End Child Process.....' + job.data.offset);
                        if (result.error) return reject();
                        resolve();
                    })
                })
                    .then(() => {
                        console.log('simulation batch completed for', job.data.simulationId, 'with offset', job.data.offset);
                        return done();
                    })
                    .catch(err => {
                        console.log(err);
                        return done(new Error('Simulation Batch Error - Queue'));
                    })
            } catch (err) {
                console.log(err);
                return done(new Error("Error in Simulations-batch"));
            }
        });

        queue.process('session', (job, done) => {
            db.Session.update({
                status: job.data.status
            }, { where: { id: job.data.sessionId } })
                .then(r => {
                    return done();
                })
                .catch(err => {
                    console.log(err);
                    return done(new Error('Session Error - Queue'));
                })
        });

        queue.process('session-event', (job, done) => {
            db.SessionEvent.create({
                simulationId: job.data.simulationId,
                sessionId: job.data.sessionId,
                message: job.data.status,
                details: job.data.details
            })
                .then(r => {
                    return done();
                })
                .catch(err => {
                    console.log(err);
                    return done(new Error('Session Error - Queue'));
                })
        });

        queue.process('session-email-open', (job, done) => {
            console.log("Email Opened", job.data.sId);
            db.Session.findOne({ where: { sessionKey: job.data.sId } })
                .then(session => {
                    if (config.value.status_priority.indexOf('Email Opened') > config.value.status_priority.indexOf(session.status))
                        return db.Session.update({ status: 'Email Opened' }, { where: { id: session.id } }).then(r => session)
                    return session;
                })
                .then(session => {
                    return db.SessionEvent.create({
                        simulationId: session.simulationId,
                        sessionId: session.id,
                        message: 'Email Opened',
                        metaData: job.data.metaData
                    });
                })
                .then(r => {
                    return done();
                })
                .catch(err => {
                    console.log(err);
                    return done(new Error('Session Error - Queue'));
                })
        });

        queue.process('session-link-clicked', (job, done) => {
            console.log("Link Clicked", job.data.sKey);
            db.Session.findOne({ where: { sessionKey: job.data.sKey } })
                .then(session => {
                    if (config.value.status_priority.indexOf('Link Clicked') > config.value.status_priority.indexOf(session.status))
                        return db.Session.update({ status: 'Link Clicked' }, { where: { id: session.id }, returning: true }).then(r => session);
                    return session;
                })
                .then(session => {
                    return db.SessionEvent.findOne({ where: { message: 'Email Opened', simulationId: session.simulationId, sessionId: session.id } }).then(r => [r, session])
                })
                .then(([event, session]) => {
                    if (!event) return db.SessionEvent.create({ simulationId: session.simulationId, sessionId: session.id, message: 'Email Opened', metaData: job.data.metaData }).then(r => session);
                    return session;
                })
                .then(session => {
                    return db.SessionEvent.create({
                        simulationId: session.simulationId,
                        sessionId: session.id,
                        message: 'Link Clicked',
                        metaData: job.data.metaData
                    })
                })
                .then(r => {
                    return done();
                })
                .catch(err => {
                    console.log(err);
                    return done(new Error('Session Error - Queue'));
                })
        });

        queue.process('create-sessions', (job, done) => {
            if (!loadAvailable) {
                console.log("task - pending");
                done(new Error('system-load'));
                return;
            }
            try {
                var data = job.data.results;
                return new Promise((resolve, reject) => {
                    const process = fork(path.join(__dirname, 'krademy/session.js'));
                    process.send({
                        simulationId: job.data.simulationId,
                        status: null,
                        groups: job.data.groups,
                        companyId: job.data.companyId,
                        isManager: job.data.isManager,
                        type: data.type
                    })
                    process.on('message', async (result) => {
                        console.log('sessions creation done....');
                        if (result.error) return reject();
                        resolve();
                    })
                })
                    .then(() => {
                        console.log('Create Session Completely Done');
                        setTimeout(function () {
                            return scheduler.scheduleSimulation(job.data.simulationId, job.data.scheduledAt, job.data.results, job.data.companyId);
                        }, 5000);
                    })
                    .then(() => {
                        return done();
                    })
                    .catch(err => {
                        console.log(err);
                        return done(new Error('Create Session---Error'));
                    })
            } catch (err) {
                console.log(err);
                return done(new Error('Session Creating Error - Queue'));
            }
        });

        queue.process('appsumo-user-package', async (job, done) => {
            try {
                var obj = [];
                db.AppsumoPlan.findOne({ where: { code: job.data.code } })
                    .then(appsumoplan => {
                        if (appsumoplan) {
                            obj.push({ companyId: job.data.companyId, name: 'users', value: appsumoplan.users });
                            obj.push({ companyId: job.data.companyId, name: 'storage', value: appsumoplan.storage });
                            obj.push({ companyId: job.data.companyId, name: 'dataTransfer', value: appsumoplan.dataTransfer });
                            return db.UserPackage.bulkCreate(obj, { returning: true });
                        }
                    })
                    .then(event => {
                        return done();
                    })
                    .catch(err => {
                        console.log(err);
                        return done(new Error("user package not updated"))
                    })
            }
            catch (err) {
                console.log(err);
                return done(new Error(err))
            }
        })

        queue.process('session-resolve-error', async (job, done) => {
            try {
                db.Session.findAll({
                    attributes: ['id', 'memberId', 'firstName', 'lastName', 'email', 'sessionKey', 'simulationId'], where: { status: 'Error', companyId: job.data.companyId, simulationId: job.data.simulationId },
                    include: [{ model: db.Member, attributes: ['code', 'courseMails', 'assessmentMails'] }, {
                        model: db.Simulation, attributes: ['type'],
                        include: [{
                            model: db.Campaign, attributes: ['id'],
                            include: [
                                { model: db.CampaignQuiz, required: false, include: [{ model: db.Quiz, attributes: ['id', 'name', 'logo'] }] },
                                {
                                    model: db.CampaignCourse, required: false, include: [{ model: db.Course, attributes: ['id', 'name', 'logo'] }
                                    ]
                                }]
                        }]
                    }],
                    limit: parseInt(job.data.limit)
                })
                    .then(result => {
                        console.log(job.data.limit, result.length);
                        if (result)
                            return sendErrorMessage(result, job.data.type);
                        else throw new RequestError('No Data found');
                    })
                    .then(mailresult => {
                        if (mailresult)
                            return mailer.sendEnrollLink(mailresult, job.data.simulationId, job.data.companyId, job.data.type);
                        else throw new RequestError('No Data found');
                    })
                    .then(r => {
                        return done();
                    })
            } catch (err) {
                console.log(err);
                return done(new Error(err))
            }
        })

        queue.process('course-reminder-mails-cron-job', async (job, done) => {
            try {
                db.Templates.findOne({ where: { name: '__Course_Reminder__', type: 'reminder' } })
                .then(template => {
                    if (template) return db.MetaData.findOne({ where: { companyId: job.data.companyId, key: template.name }, attributes: ['value'] }).then(r => [r, template])
                    else return done(new Error("Invalid Template received"));
                })
                .then(([metadata, template]) => {
                    var customData = (metadata != null ? JSON.parse(metadata.value) : '');
                    return db.Company.findOne({ where: { id: job.data.companyId }, attributes: [['id', 'companyId'], 'name', 'logo', 'smtpStatus'], order: [[db.Sender, 'id', 'DESC']], include: [{ model: db.MetaData, attributes: ['key', 'value'], where: { key: 'SDOMAIN' }, required: false }, { model: db.Sender, where: { companyId: job.data.companyId }, required: false }] }).then(r => [r, template, customData])
                })
                .then(([company, template, customData]) => {
                    if (company  && company.smtpStatus == 'custom' && company.Senders && company.Senders.length > 0) {
                        return [company.Senders[0], company, template, customData];
                    } else return db.Sender.findOne({ where: { companyId: null } }).then(r => [r, company, template, customData])
                })
                .then(([sender, company, template, customData]) => {
                    if (sender) {
                            var emailconfig = {
                                tls: { rejectUnauthorized: false },
                                host: sender.host.split(':')[0],
                                port: sender.host.split(':')[1],
                            };
                            if (sender.pool) {
                                emailconfig['pool'] = true;
                                emailconfig['maxMessages'] = sender.maxMessages;
                                emailconfig['maxConnections'] = sender.maxConnections;
                            }
                            if (sender.requiretls) {
                                emailconfig['requireTLS'] = sender.requiretls
                            } else {
                                if ((sender.ssl != null) && (sender.ssl == true || sender == false)) {
                                    emailconfig['ssl'] = sender.ssl
                                }
                            }
                            if (sender.connectionTimeout) {
                                emailconfig['connectionTimeout'] = sender.connectionTimeout
                            }
                            if (sender.username) {
                                emailconfig['auth'] = { user: sender.username, }
                                if (sender.password)
                                    return dcryptPassword(sender.password).then(r => [r, emailconfig, sender, company, template, customData])
                            }
                            return [false, emailconfig, sender, company, template, customData];
                        } else return done(new RequestError('SMTP Profile is not set.'))
                    })
                    .then(([password, emailconfig, sender, company, template, customData]) => {
                        if (password) emailconfig.auth['pass'] = password;
                        var data = { completedAt: null, courseId: { in: job.data.courses } };
                        return db.MemberCourse.count({
                            where: data,
                            include: [{
                                model: db.Session, required: true,
                                on: {
                                    memberId: db.sequelize.where(db.sequelize.col("`MemberCourse`.`memberId`"), '=', db.sequelize.col("`Session`.`memberId`"))
                                },
                                where: { simulationId: { in: job.data.simulations } }
                            }],
                        }).then(r => [r, emailconfig, sender, company, template, customData])
                    })
                    .then(([memberCounts, emailconfig, sender, company, template, customData]) => {
                        var delay = parseInt(config.mail.reminder_delay), batchingRange = config.mail.reminder_batching;
                        if (sender && company && company.smtpStatus == 'custom') {
                            if (sender.delay && sender.delay != null)
                                delay = parseInt(sender.delay);
                            if (sender.limit && sender.limit != null)
                                batchingRange = sender.limit;
                        }
                        let sessionHash = Math.random().toString(36).slice(2);
                        for (var k = 0; k < Math.ceil(memberCounts / batchingRange); k++) {
                            let offset = batchingRange * k;
                            let limit = batchingRange;
                            queue.create('send-reminder-mail-data', {
                                limit: limit,
                                offset: offset,
                                simulations: job.data.simulations,
                                courses: job.data.courses,
                                emailconfig: emailconfig,
                                fromAddress: sender.fromAddress,
                                company: company,
                                template: template,
                                customData: customData,
                                all: job.data.dataAll,
                                sessionHash: sessionHash,
                                isLast: (k == Math.ceil(memberCounts / batchingRange) - 1)
                            }).delay(delay * k).removeOnComplete(true).save();
                        }
                        return true;
                    })
                    .then(r => {
                        return done();
                    })
                    .catch(err => {
                        console.log(err);
                    })
            } catch (err) {
                console.log(err);
                return done(new Error(err))
            }
        })
  
       queue.process('send-reminder-mail-data', async (job, done) => {
            try {
                console.log(parseInt(job.data.limit), parseInt(job.data.offset));
                var data = { completedAt: null, courseId: { in: job.data.courses } };
                return db.MemberCourse.findAll({
                    attributes: ['id', 'courseId', 'quizId', 'memberId', 'sessionId', 'percent'],
                    where: data,
                    include: [
                        { model: db.Course, required: false, attributes: ['name', 'logo'] },
                        {
                            model: db.Session,
                            on: {
                                memberId: db.sequelize.where(db.sequelize.col("`MemberCourse`.`memberId`"), '=', db.sequelize.col("`Session`.`memberId`"))
                            },
                            attributes: ['id', 'memberId', 'simulationId', 'groupId', 'firstName', 'lastName', 'email', 'companyId', 'sessionKey'], required: true,
                            include: [{ model: db.Group, attributes: ['id', 'managerId'], required: true, include: [{ model: db.Member, attributes: ['id', 'firstName', 'lastName', 'email', 'code', 'reminderMails'] }] }], where: { simulationId: { in: job.data.simulations } }
                        }],
                    limit: parseInt(job.data.limit),
                    offset: parseInt(job.data.offset)
                })
                    .then(async results => {
                        let all = job.data.all;
                        let sessionHash = job.data.sessionHash;
                        var smtpTransport = nodemailer.createTransport(job.data.emailconfig);
                        if (results.length > 0) {
                            var executedlist = [], mailData = {};
                            for (let i = 0; i < results.length; i++) {
                                var simulationId = results[i].Session.simulationId;
                                var courseId = results[i].courseId;
                                var percentage = parseFloat(results[i].percent);
                                client.set(`manager-${results[i].Session.Group.id}`, JSON.stringify({ name: results[i].Session.Group.Member.firstName + " " + (results[i].Session.Group.Member.lastName ? results[i].Session.Group.Member.lastName : ""), email: results[i].Session.Group.Member.email }));
                                if (all[simulationId + "-" + courseId] && all[simulationId + "-" + courseId].length > 0) {
                                    for (var j = 0; (j < all[simulationId + "-" + courseId].length); ++j) {
                                        if (executedlist.indexOf(all[simulationId + "-" + courseId][j].id) == -1)
                                            executedlist.push(all[simulationId + "-" + courseId][j].id);
                                        if (all[simulationId + "-" + courseId][j].percent >= percentage) {
                                            client.set(`${sessionHash}-${results[i].Session.Group.id + '-' + simulationId + '-' + all[simulationId + "-" + courseId][j].id}`, JSON.stringify({ courseName: results[i].Course.name, scheduledAt: all[simulationId + "-" + courseId][j].scheduledAt, percent: all[simulationId + "-" + courseId][j].percent, companyId: results[i].Session.companyId }));
                                            let ad = { "firstName": results[i].Session.firstName, "lastName": results[i].Session.lastName, "email": results[i].Session.email, 'percentage': percentage };
                                            storeMemberInFile(sessionHash, results[i].Session.Group.id, simulationId, all[simulationId + "-" + courseId][j].id, ad)
                                            mailData = {
                                                'courseName': results[i].Course.name,
                                                'firstName': results[i].Session.firstName,
                                                'lastName': results[i].Session.lastName,
                                                'percent': percentage,
                                                // 'code': results[i].Session.Group.Member.code,
                                                'url': '/course-auth?',
                                                'courseId': courseId,
                                                'sessionKey': results[i].Session.sessionKey,
                                            }
                                            let reminderId = all[simulationId + "-" + courseId][j].id;
                                            mailer.sendCourseReminderLink(results[i].Session.email, mailData, job.data.company, job.data.template, job.data.customData, job.data.fromAddress, smtpTransport, simulationId, reminderId);
                                        }
                                    }
                                }
                            }
                            if (job.data.isLast)
                                return db.SimulationReminder.update({ executedAt: new Date() }, { where: { id: { in: executedlist } } }).then(r => sessionHash)
                        }
                        return sessionHash;
                    })
                    .then(sessionHash => {
                        if (job.data.isLast) {
                            return sendReminderToManagers(sessionHash);
                        }
                    })
                    .then(r => {
                        console.log('done------');
                        return done();
                    })
            } catch (err) {
                console.log(err)
                return done(new Error(err))
            }
        });

        queue.process('retry-remainder', async (job, done) => {
            try {
                let data = job.data.retries;
                console.log(JSON.stringify(data))
                let memberEmail = getMemberEmail(data);
                let courseData = getCourseData(data)
                let remainderData = getRemainderData(data)
                db.Member.findAll({
                    attributes: ['id', 'firstName', 'lastName', 'email', 'code', 'reminderMails'],
                    distinct: true,
                    group: ['email'],
                    where: { email: { $in: memberEmail } }
                })
                .then(member => {
                    let arr = [];
                    member.reduce((ac, element) => {
                        arr.push(element.id);
                    }, Promise.resolve());
                    return db.MemberCourse.findAll({
                        attributes: ['id', 'courseId', 'quizId', 'memberId', 'sessionId', 'percent'], include: [{
                            model: db.Session,
                            /* on: {
                                memberId: db.sequelize.where(db.sequelize.col("`MemberCourse`.`memberId`"), '=', db.sequelize.col("`Session`.`memberId`"))
                            }, */ attributes: ['id', 'memberId', 'simulationId', 'groupId', 'firstName', 'lastName', 'email', 'companyId', 'sessionKey']
                        }], where: { memberId: { $in: arr }, courseId: { $in: courseData } }
                    })
                })
                .then(courseData => {
                        return db.SimulationReminder.findAll({
                            attributes: ['id', 'type', 'simulationId', 'campaignCourseId', 'percent', 'scheduledAt'],
                            include: [{
                                model: db.CampaignCourse, attributes: ['courseId', 'campaignId'], include: [{ model: db.Course, attributes: ['id', 'name'] },
                                { model: db.Campaign, attributes: ['companyId'] }]
                            }
                            ], where: { id: { $in: remainderData }, executedAt: db.sequelize.where(db.sequelize.fn('date', db.sequelize.col('executedAt')), '=', differentDateFormat(5, undefined)) }
                        }).then(r => [r, courseData])
                })
                .then(([simulationData, courseData]) => {
                    if (simulationData.length > 0) {
                        simulationData.forEach(ele => {
                            courseData.forEach(element => {
                                if (element.courseId == ele.CampaignCourse.courseId && ele.percent >= element.percent) {
                                    console.log("queue called-----")
                                    queue.create('retry-remainder-mail-queue', {
                                        companyId: ele.CampaignCourse.Campaign.companyId,
                                        courseName: ele.CampaignCourse.Course.name,
                                        firstName: element.Session.firstName,
                                        lastName: element.Session.lastName,
                                        sessionKey: element.Session.sessionKey,
                                        courseId: ele.CampaignCourse.courseId,
                                        percent: ele.percent,
                                        email: element.Session.email,
                                        simulationId: ele.simulationId,
                                        remainderId: ele.id
                                    }).removeOnComplete(true).save();
                                }
                            })
                        })
                    }
                    return done();
                })
            } catch (err) {
                console.log(err)
                return done(new Error(err))
            }
        });//

        queue.process('retry-remainder-mail-queue', async (job, done) => {
            try {
                let companyId = job.data.companyId;
                db.Company.findOne({ where: { id: companyId },attributes:[['id','companyId'],'smtpStatus', 'name', 'logo'] })
                .then(company => {
                    if (company && company.smtpStatus == 'custom')
                        return db.Sender.findOne({ where: { companyId: companyId }, order: [['id', 'DESC']] }).then(r => [r, company])
                    else return db.Sender.findOne({ where: { companyId: null } }).then(r => [r, company])
                })
                .then(([sender, company]) => {
                    if (sender) {
                        var emailconfig = {
                            tls: { rejectUnauthorized: false },
                            host: sender.host.split(':')[0],
                            port: sender.host.split(':')[1],
                        };
                        if (sender.pool) {
                            emailconfig['pool'] = true;
                            emailconfig['maxMessages'] = sender.maxMessages;
                            emailconfig['maxConnections'] = sender.maxConnections;
                        }
                        if (sender.requiretls) {
                            emailconfig['requireTLS'] = sender.requiretls
                        } else {
                            if ((sender.ssl != null) && (sender.ssl == true || sender == false)) {
                                emailconfig['ssl'] = sender.ssl
                            }
                        }
                        if (sender.connectionTimeout) {
                            emailconfig['connectionTimeout'] = sender.connectionTimeout
                        }
                        if (sender.username) {
                            emailconfig['auth'] = { user: sender.username, }
                            if (sender.password)
                                return dcryptPassword(sender.password).then(r => [r, company, emailconfig, sender.fromAddress])
                        }
                        return [false, company, emailconfig, sender.fromAddress];
                    } else done(new RequestError('SMTP Profile is not set.'))
                })
                .then(([password, company, emailconfig, fromAddress]) => {
                    if (password) emailconfig.auth['pass'] = password;
                    return db.Templates.findOne({ where: { name: getTemplateType['reminder'], type: 'reminder' } }).then(r => [r, company, emailconfig, fromAddress])
                })
                .then(([template, company, emailconfig, fromAddress]) => {
                    if (template)
                        return db.MetaData.findOne({ where: { companyId: companyId, key: template.name }, attributes: ['value'] }).then(r => [r, company, template, emailconfig, fromAddress])
                    else done(new RequestError('No Template Selected.'))
                })
                .then(([MetaData, company, template, emailconfig, fromAddress]) => {
                    let customData = (MetaData != null ? JSON.parse(MetaData.value) : '');
                    let data = job.data;
                    let smtpTransport = nodemailer.createTransport(emailconfig);
                    let mailData = {
                        'courseName': data.courseName,
                        'firstName': data.firstName,
                        'lastName': data.lastName,
                        'percent': data.percent,
                        'url': '/course-auth?',
                        'courseId': data.courseId,
                        'sessionKey': data.sessionKey,
                    }
                    mailer.sendCourseReminderLink(data.email, mailData, company, template, customData, fromAddress, smtpTransport, data.simulationId, data.remainderId);
                    return done();
                })
            } catch (err) {
                console.log(err)
                return done(new Error(err))
            }
        });

        queue.process('remainder-resolve-error', async (job, done) => {
            try {
                let course = job.data.course;
                let lectureName = job.data.lecturename;
                let metadata = job.data.metadata;
                let template = job.data.template;
                let company = job.data.company;
                let smtpTransport = nodemailer.createTransport(job.data.emailconfig);
                let fromAddress = job.data.fromAddress;
                db.CourseUserQuizDetail.findAll({ attributes: ['id', 'courseSectionLectureId', 'sessionId', 'percent', 'attempts'], where: { courseSectionLectureId: job.data.ltd, percent: { $lt: parseInt(job.data.quiz_passing_percentage) + '.00' }, attempts: { $gte: 1 } }, include: [{ model: db.Session, attributes: ['id', 'memberId', 'firstName', 'lastName', 'email', 'sessionKey'], required: true, include: [{ model: db.Member, attributes: ['id', 'code', 'reminderMails'] }] }],
                limit: parseInt(job.data.limit),offset: parseInt(job.data.offset) })
                .then(courseDetails => {
                    if (courseDetails.length > 0) {
                        return courseDetails.reduce((p, data) => {
                            if (data.Session && data.Session.Member.reminderMails) {
                                var mailData = {
                                    firstName: data.Session.firstName,
                                    lastName: data.Session.lastName,
                                    email: data.Session.email,
                                    courseName: course.name,
                                    lectureName: lectureName,
                                    sessionKey: data.Session.sessionKey,
                                    code: data.Session.Member.code,
                                    courseOrQuiz: [{ id: course.id, name: course.name, image: course.logo }]
                                }
                                mailer.resendFailureMail(mailData,metadata,template,company,smtpTransport,fromAddress)
                            }
                        }, Promise.resolve());
                    } else return false;
                })
                .then(r => {
                    return done();
                })
            } catch (err) {
                console.log(err);
                return done(new Error(err))
            }
        })
    }
}
